import os

AUTO_LOADED_PROFILES = list(
    os.path.join(*parts)
    for parts in (
        (".landscape.yml",),
        (".landscape.yaml",),
        ("landscape.yml",),
        ("landscape.yaml",),
        (".prospector.yaml",),
        (".prospector.yml",),
        ("prospector.yaml",),
        ("prospector.yml",),
        ("prospector", ".prospector.yaml"),
        ("prospector", ".prospector.yml"),
        ("prospector", "prospector.yaml"),
        ("prospector", "prospector.yml"),
        (".prospector", ".prospector.yaml"),
        (".prospector", ".prospector.yml"),
        (".prospector", "prospector.yaml"),
        (".prospector", "prospector.yml"),
    )
)



import re
import sre_constants

import yaml
from prospector.message import Location, Message
from prospector.profiles import AUTO_LOADED_PROFILES
from prospector.tools import ToolBase, pyflakes

PROFILE_IS_EMPTY = "profile-is-empty"
CONFIG_SETTING_SHOULD_BE_LIST = "should-be-list"
CONFIG_UNKNOWN_SETTING = "unknown-setting"
CONFIG_SETTING_MUST_BE_INTEGER = "should-be-int"
CONFIG_SETTING_MUST_BE_BOOL = "should-be-bool"
CONFIG_INVALID_VALUE = "invalid-value"
CONFIG_INVALID_REGEXP = "invalid-regexp"
CONFIG_DEPRECATED_SETTING = "deprecated"
CONFIG_DEPRECATED_CODE = "deprecated-tool-code"


class ProfileValidationTool(ToolBase):

    LIST_SETTINGS = ("inherits", "uses", "ignore", "ignore-paths", "ignore-patterns")
    ALL_SETTINGS = LIST_SETTINGS + (
        "strictness",
        "autodetect",
        "max-line-length",
        "output-format",
        "output-target",
        "doc-warnings",
        "test-warnings",
        "member-warnings",
        # bit of a grim hack; prospector does not use the following but Landscape does:
        # TODO: think of a better way to avoid Landscape-specific config leaking into prospector
        "requirements",
        "python-targets",
    )

    def __init__(self):
        self.to_check = set(AUTO_LOADED_PROFILES)
        self.ignore_codes = ()

    def configure(self, prospector_config, found_files):
        for profile in prospector_config.config.profiles:
            self.to_check.add(profile)

        self.ignore_codes = prospector_config.get_disabled_messages("profile-validator")

    def tool_names(self):
        # TODO: this is currently a circular import, which is why it is not at the top of
        # the module. However, there's no obvious way to get around this right now...
        # pylint: disable=cyclic-import
        from prospector.tools import TOOLS

        return TOOLS.keys()

    def validate(self, relative_filepath, absolute_filepath):
        messages = []

        with open(absolute_filepath) as profile_file:
            raw_contents = profile_file.read()
            parsed = yaml.safe_load(raw_contents)
            raw_contents = raw_contents.split("\n")

        def add_message(code, message, setting):
            if code in self.ignore_codes:
                return
            line = -1
            for number, fileline in enumerate(raw_contents):
                if setting in fileline:
                    line = number + 1
                    break
            location = Location(relative_filepath, None, None, line, 0, False)
            message = Message("profile-validator", code, location, message)
            messages.append(message)

        if parsed is None:
            # this happens if a completely empty profile is found
            add_message(
                PROFILE_IS_EMPTY,
                "%s is a completely empty profile" % relative_filepath,
                "entire-file",
            )
            return messages

        for setting in ("doc-warnings", "test-warnings", "autodetect"):
            if not isinstance(parsed.get(setting, False), bool):
                add_message(
                    CONFIG_SETTING_MUST_BE_BOOL,
                    '"%s" should be true or false' % setting,
                    setting,
                )

        if not isinstance(parsed.get("max-line-length", 0), int):
            add_message(
                CONFIG_SETTING_MUST_BE_INTEGER,
                '"max-line-length" should be an integer',
                "max-line-length",
            )

        if "strictness" in parsed:
            possible = ("veryhigh", "high", "medium", "low", "verylow", "none")
            if parsed["strictness"] not in possible:
                add_message(
                    CONFIG_INVALID_VALUE,
                    '"strictness" must be one of %s' % ", ".join(possible),
                    "strictness",
                )

        if "uses" in parsed:
            possible = ("django", "celery", "flask")
            parsed_list = parsed["uses"] if isinstance(parsed["uses"], list) else [parsed["uses"]]
            for uses in parsed_list:
                if uses not in possible:
                    add_message(
                        CONFIG_INVALID_VALUE,
                        '"%s" is not valid for "uses", must be one of %s' % (uses, ", ".join(possible)),
                        uses,
                    )

        if "ignore" in parsed:
            add_message(
                CONFIG_DEPRECATED_SETTING,
                '"ignore" is deprecated, please update to use "ignore-patterns" instead',
                "ignore",
            )

        if "python-targets" in parsed:
            python_targets = (
                parsed["python-targets"] if isinstance(parsed["python-targets"], list) else [parsed["python-targets"]]
            )

            for target in python_targets:
                if str(target) not in ("2", "3"):
                    add_message(
                        CONFIG_INVALID_VALUE,
                        '"%s" is not valid for "python-targets", must be either 2 or 3' % target,
                        str(target),
                    )

        for pattern in parsed.get("ignore-patterns", []):
            try:
                re.compile(pattern)
            except sre_constants.error:
                add_message(CONFIG_INVALID_REGEXP, "Invalid regular expression", pattern)

        for key in ProfileValidationTool.LIST_SETTINGS:
            if key not in parsed:
                continue
            if not isinstance(parsed[key], (tuple, list)):
                add_message(CONFIG_SETTING_SHOULD_BE_LIST, '"%s" should be a list' % key, key)

        for key in parsed.keys():
            if key not in ProfileValidationTool.ALL_SETTINGS and key not in self.tool_names():
                add_message(
                    CONFIG_UNKNOWN_SETTING,
                    '"%s" is not a valid prospector setting' % key,
                    key,
                )

        if "pyflakes" in parsed:
            for code in parsed["pyflakes"].get("enable", []) + parsed["pyflakes"].get("disable", []):
                if code in pyflakes.LEGACY_CODE_MAP:
                    add_message(
                        CONFIG_DEPRECATED_CODE,
                        "Pyflakes code %s was renamed to %s" % (code, pyflakes.LEGACY_CODE_MAP[code]),
                        "pyflakes",
                    )

        return messages

    def run(self, found_files):
        messages = []
        for rel_filepath in found_files.iter_file_paths(abspath=False, include_ignored=True):
            for possible in self.to_check:
                if rel_filepath == possible:
                    abs_filepath = found_files.to_absolute_path(rel_filepath)
                    messages += self.validate(rel_filepath, abs_filepath)
                    break

        return messages



from __future__ import absolute_import

import mimetypes
import os
import re

from dodgy.checks import check_file_contents
from prospector.encoding import CouldNotHandleEncoding, read_py_file
from prospector.message import Location, Message
from prospector.tools.base import ToolBase


def module_from_path(path):
    # note : assumes a relative path
    module = re.sub(r"\.py", "", path)
    return ".".join(module.split(os.path.sep)[1:])


class DodgyTool(ToolBase):
    def run(self, found_files):

        warnings = []
        for filepath in found_files.iter_file_paths():
            mimetype = mimetypes.guess_type(filepath)
            if mimetype[0] is None or not mimetype[0].startswith("text/") or mimetype[1] is not None:
                continue
            try:
                contents = read_py_file(filepath)
            except CouldNotHandleEncoding:
                continue
            for line, code, message in check_file_contents(contents):
                warnings.append({"line": line, "code": code, "message": message, "path": filepath})

        messages = []
        for warning in warnings:
            path = warning["path"]
            prefix = os.path.commonprefix([found_files.rootpath, path])
            loc = Location(
                path,
                module_from_path(path[len(prefix) :]),
                "",
                warning["line"],
                0,
                absolute_path=True,
            )
            msg = Message("dodgy", warning["code"], loc, warning["message"])
            messages.append(msg)

        return messages



r"""
Each tool has its own method of ignoring errors and warnings.
For example, pylint requires a comment of the form

    # pylint disable=<error codes>

PEP8 will not warn on lines with

    # noqa

Additionally, flake8 follows that convention for pyflakes errors,
but pyflakes itself does not.

Finally, an entire file is ignored by flake8 if this line is found
in the file:

    # flake8\: noqa       (the \ is needed to stop prospector ignoring this file :))

This module's job is to attempt to collect all of these methods into
a single coherent list of error suppression locations.
"""
import os
import re
import warnings
from collections import defaultdict

from prospector import encoding

_FLAKE8_IGNORE_FILE = re.compile(r"flake8[:=]\s*noqa", re.IGNORECASE)
_PEP8_IGNORE_LINE = re.compile(r"#\s+noqa", re.IGNORECASE)
_PYLINT_SUPPRESSED_MESSAGE = re.compile(r"^Suppressed \'([a-z0-9-]+)\' \(from line \d+\)$")


def get_noqa_suppressions(file_contents):
    """
    Finds all pep8/flake8 suppression messages

    :param file_contents:
        A list of file lines
    :return:
        A pair - the first is whether to ignore the whole file, the
        second is a set of (0-indexed) line numbers to ignore.
    """
    ignore_whole_file = False
    ignore_lines = set()
    for line_number, line in enumerate(file_contents):
        if _FLAKE8_IGNORE_FILE.search(line):
            ignore_whole_file = True
        if _PEP8_IGNORE_LINE.search(line):
            ignore_lines.add(line_number + 1)
    return ignore_whole_file, ignore_lines


_PYLINT_EQUIVALENTS = {
    # TODO: blending has this info already?
    "unused-import": (
        ("pyflakes", "FL0001"),
        ("frosted", "E101"),
    )
}


def _parse_pylint_informational(messages):
    ignore_files = set()
    ignore_messages = defaultdict(lambda: defaultdict(list))

    for message in messages:
        if message.source == "pylint":
            if message.code == "suppressed-message":
                # this is a message indicating that a message was raised
                # by pylint but suppressed by configuration in the file
                match = _PYLINT_SUPPRESSED_MESSAGE.match(message.message)
                suppressed_code = match.group(1)
                line_dict = ignore_messages[message.location.path]
                line_dict[message.location.line].append(suppressed_code)
            elif message.code == "file-ignored":
                ignore_files.add(message.location.path)
    return ignore_files, ignore_messages


def get_suppressions(relative_filepaths, root, messages):
    """
    Given every message which was emitted by the tools, and the
    list of files to inspect, create a list of files to ignore,
    and a map of filepath -> line-number -> codes to ignore
    """
    paths_to_ignore = set()
    lines_to_ignore = defaultdict(set)
    messages_to_ignore = defaultdict(lambda: defaultdict(set))

    # first deal with 'noqa' style messages
    for filepath in relative_filepaths:
        abspath = os.path.join(root, filepath)

        try:
            file_contents = encoding.read_py_file(abspath).split("\n")
        except encoding.CouldNotHandleEncoding as err:
            # TODO: this output will break output formats such as JSON
            warnings.warn("{0}: {1}".format(err.path, err.cause), ImportWarning)
            continue

        ignore_file, ignore_lines = get_noqa_suppressions(file_contents)
        if ignore_file:
            paths_to_ignore.add(filepath)
        lines_to_ignore[filepath] |= ignore_lines

    # now figure out which messages were suppressed by pylint
    pylint_ignore_files, pylint_ignore_messages = _parse_pylint_informational(messages)
    paths_to_ignore |= pylint_ignore_files
    for filepath, line in pylint_ignore_messages.items():
        for line_number, codes in line.items():
            for code in codes:
                messages_to_ignore[filepath][line_number].add(("pylint", code))
                if code in _PYLINT_EQUIVALENTS:
                    for equivalent in _PYLINT_EQUIVALENTS[code]:
                        messages_to_ignore[filepath][line_number].add(equivalent)

    return paths_to_ignore, lines_to_ignore, messages_to_ignore



from __future__ import absolute_import

import os

from bandit.cli.main import _get_profile, _init_extensions
from bandit.core.config import BanditConfig
from bandit.core.constants import RANKING
from bandit.core.manager import BanditManager
from prospector.message import Location, Message
from prospector.tools.base import ToolBase


class BanditTool(ToolBase):
    def __init__(self, *args, **kwargs):
        super(BanditTool, self).__init__(*args, **kwargs)
        self.manager = None
        self.profile = None
        self.config_file = None
        self.agg_type = "file"
        self.severity = 0
        self.confidence = 0

    def configure(self, prospector_config, _):
        options = prospector_config.tool_options("bandit")

        if "profile" in options:
            self.profile = options["profile"]

        if "config" in options:
            self.config_file = options["config"]

        if "severity" in options:
            self.severity = options["severity"]
            if not 0 <= self.severity <= 2:
                raise ValueError("severity {!r} must be between 0 and 2".format(self.severity))

        if "confidence" in options:
            self.confidence = options["confidence"]
            if not 0 <= self.confidence <= 2:
                raise ValueError("confidence {!r} must be between 0 and 2".format(self.confidence))

        b_conf = BanditConfig(config_file=self.config_file)
        profile = _get_profile(b_conf, self.profile, self.config_file)
        extension_mgr = _init_extensions()
        extension_mgr.validate_profile(profile)

        self.manager = BanditManager(b_conf, None, profile=profile)

    def run(self, found_files):
        self.manager.files_list = sorted(found_files.iter_file_paths())
        self.manager.exclude_files = []

        if not self.manager.b_ts.tests:
            raise ValueError("No test will run for bandit")

        self.manager.run_tests()
        results = self.manager.get_issue_list(sev_level=RANKING[self.severity], conf_level=RANKING[self.confidence])
        messages = []
        for result in results:
            loc = Location(os.path.abspath(result.fname), None, "", int(result.lineno), 0)
            msg = Message("bandit", result.test_id, loc, result.text)
            messages.append(msg)
        return messages



from __future__ import absolute_import

from prospector.message import Location, Message
from prospector.tools.base import ToolBase
from pyflakes.api import checkPath
from pyflakes.reporter import Reporter

__all__ = ("PyFlakesTool",)


# Prospector uses the same pyflakes codes as flake8 defines,
# see https://flake8.pycqa.org/en/latest/user/error-codes.html
# and https://gitlab.com/pycqa/flake8/-/blob/e817c63a/src/flake8/plugins/pyflakes.py
_MESSAGE_CODES = {
    "UnusedImport": "F401",
    "ImportShadowedByLoopVar": "F402",
    "ImportStarUsed": "F403",
    "LateFutureImport": "F404",
    "ImportStarUsage": "F405",
    "ImportStarNotPermitted": "F406",
    "FutureFeatureNotDefined": "F407",
    "PercentFormatInvalidFormat": "F501",
    "PercentFormatExpectedMapping": "F502",
    "PercentFormatExpectedSequence": "F503",
    "PercentFormatExtraNamedArguments": "F504",
    "PercentFormatMissingArgument": "F505",
    "PercentFormatMixedPositionalAndNamed": "F506",
    "PercentFormatPositionalCountMismatch": "F507",
    "PercentFormatStarRequiresSequence": "F508",
    "PercentFormatUnsupportedFormatCharacter": "F509",
    "StringDotFormatInvalidFormat": "F521",
    "StringDotFormatExtraNamedArguments": "F522",
    "StringDotFormatExtraPositionalArguments": "F523",
    "StringDotFormatMissingArgument": "F524",
    "StringDotFormatMixingAutomatic": "F525",
    "FStringMissingPlaceholders": "F541",
    "MultiValueRepeatedKeyLiteral": "F601",
    "MultiValueRepeatedKeyVariable": "F602",
    "TooManyExpressionsInStarredAssignment": "F621",
    "TwoStarredExpressions": "F622",
    "AssertTuple": "F631",
    "IsLiteral": "F632",
    "InvalidPrintSyntax": "F633",
    "IfTuple": "F634",
    "BreakOutsideLoop": "F701",
    "ContinueOutsideLoop": "F702",
    "ContinueInFinally": "F703",
    "YieldOutsideFunction": "F704",
    "ReturnWithArgsInsideGenerator": "F705",
    "ReturnOutsideFunction": "F706",
    "DefaultExceptNotLast": "F707",
    "DoctestSyntaxError": "F721",
    "ForwardAnnotationSyntaxError": "F722",
    "CommentAnnotationSyntaxError": "F723",
    "Redefined": "F810",
    "RedefinedWhileUnused": "F811",
    "RedefinedInListComp": "F812",
    "UndefinedName": "F821",
    "UndefinedExport": "F822",
    "UndefinedLocal": "F823",
    "DuplicateArgument": "F831",
    "UnusedVariable": "F841",
    "RaiseNotImplemented": "F901",
}

# The old prospector codes were deprecated in favour of using the codes
# defined by flake8. This maps the old codes to the new codes.
LEGACY_CODE_MAP = {
    "FL0001": "F401",
    "FL0002": "F811",
    "FL0003": "F812",
    "FL0004": "F402",
    "FL0005": "F403",
    "FL0006": "F821",
    "FL0008": "F822",
    "FL0009": "F823",
    "FL0010": "F831",
    "FL0011": "F810",
    "FL0012": "F404",
    "FL0013": "F841",
}


class ProspectorReporter(Reporter):
    def __init__(self, ignore=None):
        super(ProspectorReporter, self).__init__(None, None)
        self._messages = []
        self.ignore = ignore or ()

    # pylint: disable=too-many-arguments
    def record_message(self, filename=None, line=None, character=None, code=None, message=None):

        code = code or "F999"
        if code in self.ignore:
            return

        location = Location(
            path=filename,
            module=None,
            function=None,
            line=line,
            character=character,
        )
        message = Message(
            source="pyflakes",
            code=code,
            location=location,
            message=message,
        )
        self._messages.append(message)

    def unexpectedError(self, filename, msg):  # noqa
        self.record_message(
            filename=filename,
            code="F999",
            message=msg,
        )

    # pylint: disable=too-many-arguments
    def syntaxError(self, filename, msg, lineno, offset, text):  # noqa
        self.record_message(
            filename=filename,
            line=lineno,
            character=offset,
            code="F999",
            message=msg,
        )

    def flake(self, message):
        code = _MESSAGE_CODES.get(message.__class__.__name__, "F999")

        self.record_message(
            filename=message.filename,
            line=message.lineno,
            character=(message.col + 1),
            code=code,
            message=message.message % message.message_args,
        )

    def get_messages(self):
        return self._messages


class PyFlakesTool(ToolBase):
    def __init__(self, *args, **kwargs):
        super(PyFlakesTool, self).__init__(*args, **kwargs)
        self.ignore_codes = ()

    def configure(self, prospector_config, _):
        ignores = prospector_config.get_disabled_messages("pyflakes")
        # convert old style to new
        self.ignore_codes = [LEGACY_CODE_MAP.get(code, code) for code in ignores]

    def run(self, found_files):
        reporter = ProspectorReporter(ignore=self.ignore_codes)
        for filepath in found_files.iter_module_paths():
            checkPath(filepath, reporter)

        return reporter.get_messages()



from __future__ import absolute_import

from pylint.config import OptionsManagerMixIn
from pylint.lint import PyLinter
from pylint.utils import _splitstrip


class ProspectorLinter(PyLinter):  # pylint: disable=too-many-ancestors,too-many-public-methods
    def __init__(self, found_files, *args, **kwargs):
        self._files = found_files
        # set up the standard PyLint linter
        PyLinter.__init__(self, *args, **kwargs)

    def config_from_file(self, config_file=None):
        """Will return `True` if plugins have been loaded. For pylint>=1.5. Else `False`."""
        self.read_config_file(config_file)
        if self.cfgfile_parser.has_option("MASTER", "load-plugins"):
            # pylint: disable=protected-access
            plugins = _splitstrip(self.cfgfile_parser.get("MASTER", "load-plugins"))
            self.load_plugin_modules(plugins)
        self.load_config_file()
        return True

    def reset_options(self):
        # for example, we want to re-initialise the OptionsManagerMixin
        # to suppress the config error warning
        # pylint: disable=non-parent-init-called
        OptionsManagerMixIn.__init__(self, usage=PyLinter.__doc__)

    def _expand_files(self, modules):
        expanded = PyLinter._expand_files(self, modules)
        filtered = []
        for module in expanded:
            if self._files.check_module(module["path"]):
                filtered.append(module)
        return filtered



from __future__ import absolute_import

import json
from datetime import datetime

from prospector.formatters.base import Formatter

__all__ = ("JsonFormatter",)


# pylint: disable=too-few-public-methods
class JsonFormatter(Formatter):
    def render(self, summary=True, messages=True, profile=False):
        output = {}

        if summary:
            # we need to slightly change the types and format
            # of a few of the items in the summary to make
            # them play nice with JSON formatting
            munged = {}
            for key, value in self.summary.items():
                if isinstance(value, datetime):
                    munged[key] = str(value)
                else:
                    munged[key] = value
            output["summary"] = munged

        if profile:
            output["profile"] = self.profile.as_dict()

        for message in self.messages:
            if not isinstance(message.location.path, str):
                message.location.path = str(message.location.path)
        if messages:
            output["messages"] = [m.as_dict() for m in self.messages]

        return json.dumps(output, indent=2)



import os


def is_virtualenv(path):
    if os.name == "nt":
        # Windows!
        clues = ("Scripts", "lib", "include")
    else:
        clues = ("bin", "lib", "include")

    try:
        dircontents = os.listdir(path)
    except (OSError, TypeError):
        # listdir failed, probably due to path length issues in windows
        return False

    if not all([clue in dircontents for clue in clues]):
        # we don't have the 3 directories which would imply
        # this is a virtualenvironment
        return False

    if not all([os.path.isdir(os.path.join(path, clue)) for clue in clues]):
        # some of them are not actually directories
        return False

    # if we do have all three directories, make sure that it's not
    # just a coincidence by doing some heuristics on the rest of
    # the directory
    if len(dircontents) > 7:
        # if there are more than 7 things it's probably not a virtualenvironment
        return False

    return True



from __future__ import absolute_import

import ast

from mccabe import PathGraphingAstVisitor
from prospector.encoding import CouldNotHandleEncoding, read_py_file
from prospector.message import Location, Message, make_tool_error_message
from prospector.tools.base import ToolBase

__all__ = ("McCabeTool",)


class McCabeTool(ToolBase):
    def __init__(self, *args, **kwargs):
        super(McCabeTool, self).__init__(*args, **kwargs)
        self.ignore_codes = ()
        self.max_complexity = 10

    def configure(self, prospector_config, _):
        self.ignore_codes = prospector_config.get_disabled_messages("mccabe")

        options = prospector_config.tool_options("mccabe")
        if "max-complexity" in options:
            self.max_complexity = options["max-complexity"]

    def run(self, found_files):
        messages = []

        for code_file in found_files.iter_module_paths():
            try:
                contents = read_py_file(code_file)
                tree = ast.parse(
                    contents,
                    filename=code_file,
                )
            except CouldNotHandleEncoding as err:
                messages.append(
                    make_tool_error_message(
                        code_file,
                        "mccabe",
                        "MC0000",
                        message="Could not handle the encoding of this file: %s" % err.encoding,
                    )
                )
                continue
            except SyntaxError as err:
                messages.append(
                    make_tool_error_message(
                        code_file,
                        "mccabe",
                        "MC0000",
                        line=err.lineno,
                        character=err.offset,
                        message="Syntax Error",
                    )
                )
                continue
            except TypeError:
                messages.append(make_tool_error_message(code_file, "mccabe", "MC0000", message="Unable to parse file"))
                continue

            visitor = PathGraphingAstVisitor()
            visitor.preorder(tree, visitor)

            for graph in visitor.graphs.values():
                complexity = graph.complexity()
                if complexity > self.max_complexity:
                    location = Location(
                        path=code_file,
                        module=None,
                        function=graph.entity,
                        line=graph.lineno,
                        character=0,
                        absolute_path=True,
                    )
                    message = Message(
                        source="mccabe",
                        code="MC0001",
                        location=location,
                        message="%s is too complex (%s)"
                        % (
                            graph.entity,
                            complexity,
                        ),
                    )
                    messages.append(message)

        return self.filter_messages(messages)

    def filter_messages(self, messages):
        return [message for message in messages if message.code not in self.ignore_codes]



import os
import re

from prospector.formatters.base import Formatter


class PylintFormatter(Formatter):

    """
    This formatter outputs messages in the same way as pylint -f parseable , which is used by several
    tools to parse pylint output. This formatter is therefore a compatibility shim between tools built
    on top of pylint and prospector itself.
    """

    def render(self, summary=True, messages=True, profile=False):
        # this formatter will always ignore the summary and profile
        cur_loc = None
        output = []
        for message in sorted(self.messages):
            if cur_loc != message.location.path:
                cur_loc = message.location.path
                module_name = message.location.path.replace(os.path.sep, ".")
                module_name = re.sub(r"(\.__init__)?\.py$", "", module_name)

                header = "************* Module %s" % module_name
                output.append(header)

            #   ={path}:{line}: [{msg_id}({symbol}), {obj}] {msg}
            # prospector/configuration.py:65: [missing-docstring(missing-docstring), build_default_sources] \
            # Missing function docstring

            template = "%(path)s:%(line)s: [%(code)s(%(source)s), %(function)s] %(message)s"
            output.append(
                template
                % {
                    "path": message.location.path,
                    "line": message.location.line,
                    "source": message.source,
                    "code": message.code,
                    "function": message.location.function,
                    "message": message.message.strip(),
                }
            )

        return "\n".join(output)



from prospector.encoding import CouldNotHandleEncoding, read_py_file
from prospector.message import Location, Message, make_tool_error_message
from prospector.tools.base import ToolBase
from vulture import Vulture


class ProspectorVulture(Vulture):
    def __init__(self, found_files):
        Vulture.__init__(self, verbose=False)
        self._files = found_files
        self._internal_messages = []

    def scavenge(self, _=None):
        # The argument is a list of paths, but we don't care
        # about that as we use the found_files object. The
        # argument is here to explicitly acknowledge that we
        # are overriding the Vulture.scavenge method.
        for module in self._files.iter_module_paths():
            try:
                module_string = read_py_file(module)
            except CouldNotHandleEncoding as err:
                self._internal_messages.append(
                    make_tool_error_message(
                        module,
                        "vulture",
                        "V000",
                        message="Could not handle the encoding of this file: %s" % err.encoding,
                    )
                )
                continue
            self.file = module
            self.filename = module
            try:
                self.scan(module_string, filename=module)
            except TypeError:
                self.scan(module_string)

    def get_messages(self):
        all_items = (
            ("unused-function", "Unused function %s", self.unused_funcs),
            ("unused-property", "Unused property %s", self.unused_props),
            ("unused-variable", "Unused variable %s", self.unused_vars),
            ("unused-attribute", "Unused attribute %s", self.unused_attrs),
        )

        vulture_messages = []
        for code, template, items in all_items:
            for item in items:
                try:
                    filename = item.file
                except AttributeError:
                    filename = item.filename
                if hasattr(item, "lineno"):
                    lineno = item.lineno  # for older versions of vulture
                else:
                    lineno = item.first_lineno
                loc = Location(filename, None, None, lineno, -1)
                message_text = template % item
                message = Message("vulture", code, loc, message_text)
                vulture_messages.append(message)

        return self._internal_messages + vulture_messages


class VultureTool(ToolBase):
    def __init__(self):
        ToolBase.__init__(self)
        self._vulture = None
        self.ignore_codes = ()

    def configure(self, prospector_config, found_files):
        self.ignore_codes = prospector_config.get_disabled_messages("vulture")

    def run(self, found_files):
        vulture = ProspectorVulture(found_files)
        vulture.scavenge()
        return [message for message in vulture.get_messages() if message.code not in self.ignore_codes]



import os
import re
import sys

from setoptconf.datatype import Choice


class OutputChoice(Choice):
    def sanitize(self, value):
        parsed = re.split(r"[;:]", value)
        output_format, output_targets = parsed[0], parsed[1:]
        checked_targets = []
        for i, target in enumerate(output_targets):
            if sys.platform.startswith("win") and target.startswith((os.path.sep, os.path.altsep)):
                checked_targets[-1] += ":" + target
            else:
                checked_targets.append(target)
        validated_format = super(OutputChoice, self).sanitize(output_format)
        return (validated_format, checked_targets)



import os
import re
import warnings

from prospector import encoding
from prospector.pathutils import is_virtualenv
from requirements_detector import find_requirements
from requirements_detector.detect import RequirementsNotFound

POSSIBLE_LIBRARIES = ("django", "celery", "flask")


# see http://docs.python.org/2/reference/lexical_analysis.html#identifiers
_FROM_IMPORT_REGEX = re.compile(r"^\s*from ([\._a-zA-Z0-9]+) import .*$")
_IMPORT_REGEX = re.compile(r"^\s*import ([\._a-zA-Z0-9]+)$")
_IMPORT_MULTIPLE_REGEX = re.compile(r"^\s*import ([\._a-zA-Z0-9]+(, ){1})+")


def find_from_imports(file_contents):
    names = set()
    for line in file_contents.split("\n"):
        match = _IMPORT_MULTIPLE_REGEX.match(line)
        if match:
            import_names = []
            first = match.group(1)
            import_names.append(first[:-2])
            for name in line.split(first)[1].split(","):
                import_names.append(name.strip())
        else:
            match = _IMPORT_REGEX.match(line) or _FROM_IMPORT_REGEX.match(line)
            if match is None:
                continue
            import_names = match.group(1).split(".")

        for import_name in import_names:
            if import_name in POSSIBLE_LIBRARIES:
                names.add(import_name)

    return names


def find_from_path(path):
    names = set()
    max_possible = len(POSSIBLE_LIBRARIES)

    for item in os.listdir(path):
        item_path = os.path.abspath(os.path.join(path, item))
        if os.path.isdir(item_path):
            if is_virtualenv(item_path):
                continue
            names |= find_from_path(item_path)
        elif not os.path.islink(item_path) and item_path.endswith(".py"):
            try:
                contents = encoding.read_py_file(item_path)
                names |= find_from_imports(contents)
            except encoding.CouldNotHandleEncoding as err:
                # TODO: this output will break output formats such as JSON
                warnings.warn("{0}: {1}".format(err.path, err.cause), ImportWarning)

        if len(names) == max_possible:
            # don't continue on recursing, there's no point!
            break

    return names


def find_from_requirements(path):
    reqs = find_requirements(path)
    names = []
    for requirement in reqs:
        if requirement.name is not None and requirement.name.lower() in POSSIBLE_LIBRARIES:
            names.append(requirement.name.lower())
    return names


def autodetect_libraries(path):

    if os.path.isfile(path):
        path = os.path.dirname(path)
        if path == "":
            path = "."

    libraries = []

    try:
        libraries = find_from_requirements(path)

    # pylint: disable=pointless-except
    except RequirementsNotFound:
        pass

    if len(libraries) < len(POSSIBLE_LIBRARIES):
        libraries = find_from_path(path)

    return libraries



from prospector.formatters.text import TextFormatter

__all__ = ("EmacsFormatter",)


class EmacsFormatter(TextFormatter):
    def render_message(self, message):
        output = [
            "%s:%s:%d:"
            % (
                message.location.path,
                message.location.line,
                (message.location.character or 0) + 1,
            ),
            "    L%s:%s %s: %s - %s"
            % (
                message.location.line or "-",
                message.location.character if message.location.line else "-",
                message.location.function,
                message.source,
                message.code,
            ),
            "    %s" % message.message,
        ]

        return "\n".join(output)



class ToolBase(object):
    # This is an 'abstract' base class, used to provide an indication of
    # how to create a new tool class. Therefore, the arguments will be unused,
    # so that is suppressed here.
    # pylint: disable=unused-argument

    def configure(self, prospector_config, found_files):
        """
        Tools have their own way of being configured from configuration files
        on the current path - for example, a .pep8rc file. Prospector will use
        its own configuration settings unless this method discovers some
        tool-specific configuration that should be used instead.

        :return: A tuple: the first element is a string indicating how or where
                 this tool was configured from. For example, this can be a path
                 to the .pylintrc file used, if used. None means that prospector
                 defaults were used. The second element should be an iterable of
                 Message objects representing any issues which were found when
                 trying to load configuration - for example, bad values in a
                 .pylintrc file. It is also possible to simply return None if
                 neither value is useful.
        """
        # We don't want Pylint to tell us that this method should be a function,
        # it's an "abstract" class after all.
        # pylint: disable=no-self-use
        return None, None

    def run(self, found_files):
        """
        Actually run the tool and collect the various messages emitted by the tool.
        It is expected that this will convert whatever output of the tool into the
        standard prospector Message and Location objects.
        """
        raise NotImplementedError



from prospector.formatters.base import Formatter

__all__ = ("TextFormatter",)


# pylint: disable=unnecessary-lambda


class TextFormatter(Formatter):
    summary_labels = (
        ("started", "Started"),
        ("completed", "Finished"),
        ("time_taken", "Time Taken", lambda x: "%s seconds" % x),
        ("formatter", "Formatter"),
        ("profiles", "Profiles"),
        ("strictness", "Strictness"),
        ("libraries", "Libraries Used", lambda x: ", ".join(x)),
        ("tools", "Tools Run", lambda x: ", ".join(x)),
        ("adaptors", "Adaptors", lambda x: ", ".join(x)),
        ("message_count", "Messages Found"),
        ("external_config", "External Config"),
    )

    def render_summary(self):
        output = [
            "Check Information",
            "=================",
        ]

        label_width = max([len(label[1]) for label in self.summary_labels])

        for summary_label in self.summary_labels:
            key = summary_label[0]
            if key in self.summary:
                label = summary_label[1]
                if len(summary_label) > 2:
                    value = summary_label[2](self.summary[key])
                else:
                    value = self.summary[key]
                output.append(
                    " %s: %s"
                    % (
                        label.rjust(label_width),
                        value,
                    )
                )

        return "\n".join(output)

    # pylint: disable=no-self-use
    def render_message(self, message):
        output = []

        if message.location.module:
            output.append("%s (%s):" % (message.location.module, message.location.path))
        else:
            output.append("%s:" % message.location.path)

        output.append(
            "    L%s:%s %s: %s - %s"
            % (
                message.location.line or "-",
                message.location.character if message.location.character else "-",
                message.location.function,
                message.source,
                message.code,
            )
        )

        output.append("    %s" % message.message)

        return "\n".join(output)

    def render_messages(self):
        output = [
            "Messages",
            "========",
            "",
        ]

        for message in self.messages:
            output.append(self.render_message(message))
            output.append("")

        return "\n".join(output)

    def render_profile(self):
        output = ["Profile", "=======", "", self.profile.as_yaml().strip()]

        return "\n".join(output)

    def render(self, summary=True, messages=True, profile=False):
        output = []
        if messages and self.messages:  # if there are no messages, don't render an empty header
            output.append(self.render_messages())
        if profile:
            output.append(self.render_profile())
        if summary:
            output.append(self.render_summary())

        return "\n\n\n".join(output) + "\n"



from __future__ import absolute_import

from prospector.message import Location, Message
from pylint.exceptions import UnknownMessageError
from pylint.reporters import BaseReporter


class Collector(BaseReporter):

    name = "collector"

    def __init__(self, message_store):
        BaseReporter.__init__(self, output=None)
        self._message_store = message_store
        self._messages = []

    def handle_message(self, msg):
        location = (msg.abspath, msg.module, msg.obj, msg.line, msg.column)
        self.add_message(msg.msg_id, location, msg.msg)

    def add_message(self, msg_id, location, msg):
        # (* magic is acceptable here)
        loc = Location(*location)
        # At this point pylint will give us the code but we want the
        # more user-friendly symbol
        try:
            msg_data = self._message_store.get_message_definitions(msg_id)
        except UnknownMessageError:
            # this shouldn't happen, as all pylint errors should be
            # in the message store, but just in case we'll fall back
            # to using the code.
            msg_symbol = msg_id
        else:
            msg_symbol = msg_data[0].symbol

        message = Message("pylint", msg_symbol, loc, msg)
        self._messages.append(message)

    def _display(self, layout):
        pass

    def get_messages(self):
        return self._messages



from prospector.formatters import emacs, grouped, json, pylint, text, vscode, xunit, yaml

__all__ = ("FORMATTERS",)


FORMATTERS = {
    "json": json.JsonFormatter,
    "text": text.TextFormatter,
    "grouped": grouped.GroupedFormatter,
    "emacs": emacs.EmacsFormatter,
    "yaml": yaml.YamlFormatter,
    "pylint": pylint.PylintFormatter,
    "xunit": xunit.XunitFormatter,
    "vscode": vscode.VSCodeFormatter,
}



from mypy import api
from prospector.message import Location, Message
from prospector.tools import ToolBase

__all__ = ("MypyTool",)


MYPY_OPTIONS = ["allow", "check", "disallow", "no-check", "no-warn", "warn"]


def format_message(message):
    try:
        (path, line, char, err_type, err_msg) = message.split(":", 4)
        line = int(line)
        character = int(char)
    except ValueError:
        try:
            (path, line, err_type, err_msg) = message.split(":", 3)
            line = int(line)
            character = None
        except ValueError:
            (path, err_type, err_msg) = message.split(":", 2)
            line = 0
            character = None
    location = Location(
        path=path,
        module=None,
        function=None,
        line=line,
        character=character,
    )
    return Message(
        source="mypy",
        code=err_type.lstrip(" "),
        location=location,
        message=err_msg.lstrip(" "),
    )


class MypyTool(ToolBase):
    def __init__(self, *args, **kwargs):
        super(MypyTool, self).__init__(*args, **kwargs)
        self.checker = api
        self.options = ["--show-column-numbers", "--no-error-summary"]

    def configure(self, prospector_config, _):
        options = prospector_config.tool_options("mypy")

        strict = options.get("strict", False)

        follow_imports = options.get("follow-imports", "normal")
        ignore_missing_imports = options.get("ignore-missing-imports", False)
        implict_optional = options.get("implict-optional", False)
        platform = options.get("platform", None)
        python_2_mode = options.get("python-2-mode", False)
        python_version = options.get("python-version", None)
        strict_optional = options.get("strict-optional", False)
        namespace_packages = options.get("namespace-packages", False)

        self.options.append("--follow-imports=%s" % follow_imports)

        if strict:
            self.options.append("--strict")

        if ignore_missing_imports:
            self.options.append("--ignore-missing-imports")

        if implict_optional:
            self.options.append("--implict-optional")

        if platform:
            self.options.append("--platform %s" % platform)

        if python_2_mode:
            self.options.append("--py2")

        if python_version:
            self.options.append("--python-version %s" % python_version)

        if strict_optional:
            self.options.append("--strict-optional")

        if namespace_packages:
            self.options.append("--namespace-packages")

        for list_option in MYPY_OPTIONS:
            for entry in options.get(list_option, []):
                self.options.append("--%s-%s" % (list_option, entry))

    def run(self, found_files):
        paths = [path for path in found_files.iter_module_paths()]
        paths.extend(self.options)
        result = self.checker.run(paths)
        report, _ = result[0], result[1:]  # noqa

        return [format_message(message) for message in report.splitlines()]



from xml.dom.minidom import Document

from prospector.formatters.base import Formatter


class XunitFormatter(Formatter):

    """
    This formatter outputs messages in the Xunit xml format, which is used by several
    CI tools to parse output. This formatter is therefore a compatibility shim between tools built
    to use Xunit and prospector itself.
    """

    def render(self, summary=True, messages=True, profile=False):
        xml_doc = Document()

        testsuite_el = xml_doc.createElement("testsuite")
        testsuite_el.setAttribute("errors", str(self.summary["message_count"]))
        testsuite_el.setAttribute("failures", "0")
        testsuite_el.setAttribute("name", "prospector-%s" % "-".join(self.summary["tools"]))
        testsuite_el.setAttribute("tests", str(self.summary["message_count"]))
        testsuite_el.setAttribute("time", str(self.summary["time_taken"]))
        xml_doc.appendChild(testsuite_el)

        prop_el = xml_doc.createElement("properties")
        testsuite_el.appendChild(prop_el)

        sysout_el = xml_doc.createElement("system-out")
        sysout_el.appendChild(xml_doc.createCDATASection(""))
        testsuite_el.appendChild(sysout_el)

        syserr_el = xml_doc.createElement("system-err")
        syserr_el.appendChild(xml_doc.createCDATASection(""))
        testsuite_el.appendChild(syserr_el)

        for message in sorted(self.messages):
            testcase_el = xml_doc.createElement("testcase")
            testcase_el.setAttribute("name", "%s-%s" % (message.location.path, message.location.line))

            failure_el = xml_doc.createElement("error")
            failure_el.setAttribute("message", message.message.strip())
            failure_el.setAttribute("type", "%s Error" % message.source)
            template = "%(path)s:%(line)s: [%(code)s(%(source)s), %(function)s] %(message)s"
            cdata = template % {
                "path": message.location.path,
                "line": message.location.line,
                "source": message.source,
                "code": message.code,
                "function": message.location.function,
                "message": message.message.strip(),
            }
            failure_el.appendChild(xml_doc.createCDATASection(cdata))

            testcase_el.appendChild(failure_el)

            testsuite_el.appendChild(testcase_el)

        return xml_doc.toprettyxml()



import os

from prospector.suppression import get_suppressions


def filter_messages(relative_filepaths, root, messages):
    """
    This method post-processes all messages output by all tools, in order to filter
    out any based on the overall output.

    The main aim currently is to use information about messages suppressed by
    pylint due to inline comments, and use that to suppress messages from other
    tools representing the same problem.

    For example:

        import banana  # pylint:disable=unused-import

    In this situation, pylint will not warn about an unused import as there is
    inline configuration to disable the warning. Pyflakes will still raise that
    error, however, because it does not understand pylint disabling messages.
    This method uses the information about suppressed messages from pylint to
    squash the unwanted redundant error from pyflakes and frosted.
    """
    paths_to_ignore, lines_to_ignore, messages_to_ignore = get_suppressions(relative_filepaths, root, messages)

    filtered = []
    for message in messages:
        # first get rid of the pylint informational messages
        relative_message_path = os.path.relpath(message.location.path)

        if message.source == "pylint" and message.code in (
            "suppressed-message",
            "file-ignored",
        ):
            continue

        # some files are skipped entirely by messages
        if relative_message_path in paths_to_ignore:
            continue

        # some lines are skipped entirely by messages
        if relative_message_path in lines_to_ignore:
            if message.location.line in lines_to_ignore[relative_message_path]:
                continue

        # and some lines have only certain messages explicitly ignored
        if relative_message_path in messages_to_ignore:
            if message.location.line in messages_to_ignore[relative_message_path]:
                if message.code in messages_to_ignore[relative_message_path][message.location.line]:
                    continue

        # otherwise this message was not filtered
        filtered.append(message)

    return filtered



from __future__ import absolute_import

import os
import re

from pep8ext_naming import NamingChecker
from prospector.message import Location, Message
from prospector.tools.base import ToolBase
from pycodestyle import PROJECT_CONFIG, BaseReport, StyleGuide, register_check

try:
    # for pep8 <= 1.5.7
    from pycodestyle import DEFAULT_CONFIG as USER_CONFIG
except ImportError:
    # for pep8 >= 1.6.0
    from pycodestyle import USER_CONFIG


__all__ = ("Pep8Tool",)


class ProspectorReport(BaseReport):
    def __init__(self, *args, **kwargs):
        super(ProspectorReport, self).__init__(*args, **kwargs)
        self._prospector_messages = []

    def error(self, line_number, offset, text, check):
        code = super(ProspectorReport, self).error(
            line_number,
            offset,
            text,
            check,
        )
        if code is None:
            # The error pep8 found is being ignored, let's move on.
            return
        else:
            # Get a clean copy of the message text without the code embedded.
            text = text[5:]

        # mixed indentation (E101) is a file global message
        if code == "E101":
            line_number = None

        # Record the message using prospector's data structures.
        location = Location(
            path=self.filename,
            module=None,
            function=None,
            line=line_number,
            character=(offset + 1),
        )
        message = Message(
            source="pep8",
            code=code,
            location=location,
            message=text,
        )

        self._prospector_messages.append(message)

    def get_messages(self):
        return self._prospector_messages


class ProspectorStyleGuide(StyleGuide):
    def __init__(self, found_files, *args, **kwargs):
        self._files = found_files
        self._module_paths = set(self._files.iter_module_paths())

        # Override the default reporter with our custom one.
        kwargs["reporter"] = ProspectorReport

        super(ProspectorStyleGuide, self).__init__(*args, **kwargs)

    def excluded(self, filename, parent=None):
        if super(ProspectorStyleGuide, self).excluded(filename, parent):
            return True

        # If the file survived pep8's exclusion rules, check it against
        # prospector's patterns.
        fullpath = os.path.join(self._files.rootpath, parent or "", filename)
        if os.path.isdir(fullpath):
            return False

        fullpath = fullpath if parent else filename
        return fullpath not in self._module_paths


class Pep8Tool(ToolBase):
    def __init__(self, *args, **kwargs):
        super(Pep8Tool, self).__init__(*args, **kwargs)
        self.checker = None

    def configure(self, prospector_config, found_files):
        # figure out if we should use a pre-existing config file
        # such as setup.cfg or tox.ini
        external_config = None

        # 'none' means we ignore any external config, so just carry on
        use_config = False

        if prospector_config.use_external_config("pep8"):
            use_config = True

            paths = [os.path.join(found_files.rootpath, name) for name in PROJECT_CONFIG]
            paths.append(USER_CONFIG)
            ext_loc = prospector_config.external_config_location("pep8")
            if ext_loc is not None:
                paths = [ext_loc] + paths

            for conf_path in paths:
                if os.path.exists(conf_path) and os.path.isfile(conf_path):
                    # this file exists - but does it have pep8 or pycodestyle config in it?
                    header = re.compile(r"\[(pep8|pycodestyle)\]")
                    with open(conf_path) as conf_file:
                        if any([header.search(line) for line in conf_file.readlines()]):
                            external_config = conf_path
                            break

        # create a list of packages, but don't include packages which are
        # subpackages of others as checks will be duplicated
        packages = [os.path.split(p) for p in found_files.iter_package_paths(abspath=False)]
        packages.sort(key=len)
        check_paths = set()
        for package in packages:
            package_path = os.path.join(*package)
            if len(package) == 1:
                check_paths.add(package_path)
                continue
            if os.path.join(*package) in check_paths:
                continue
            check_paths.add(package_path)

        for filepath in found_files.iter_module_paths(abspath=False):
            package = os.path.dirname(filepath).split(os.path.sep)
            for i in range(0, len(package)):
                if os.path.join(*package[: i + 1]) in check_paths:
                    break
            else:
                check_paths.add(filepath)

        check_paths = [found_files.to_absolute_path(p) for p in check_paths]

        # Instantiate our custom pep8 checker.
        self.checker = ProspectorStyleGuide(paths=check_paths, found_files=found_files, config_file=external_config)
        if not use_config or external_config is None:
            configured_by = None
            # This means that we don't have existing config to use.
            # Make sure pep8's code ignores are fully reset to zero before
            # adding prospector-flavoured configuration.
            # pylint: disable=attribute-defined-outside-init
            self.checker.options.select = ()
            self.checker.options.ignore = tuple(prospector_config.get_disabled_messages("pep8"))

            if "max-line-length" in prospector_config.tool_options("pep8"):
                self.checker.options.max_line_length = prospector_config.tool_options("pep8")["max-line-length"]
        else:
            configured_by = "Configuration found at %s" % external_config

        # if we have a command line --max-line-length argument, that
        # overrules everything
        max_line_length = prospector_config.max_line_length
        if max_line_length is not None:
            self.checker.options.max_line_length = max_line_length

        return configured_by, []

    def run(self, _):
        report = self.checker.check_files()
        return report.get_messages()


# Load pep8ext_naming into pep8's configuration.
register_check(NamingChecker)



# We are trying to handle pylint changes in their exception classes


class FatalProspectorException(Exception):

    """
    Exception used to indicate an internal prospector problem.
    Problems in prospector itself should raise this to notify
    the user directly. Errors in dependent tools which should
    be squashed and the user notified elegantly.
    """

    def __init__(self, message):
        self.message = message



from __future__ import absolute_import

import os.path
import sys
from datetime import datetime

from prospector import blender, postfilter, tools
from prospector.config import ProspectorConfig
from prospector.config import configuration as cfg
from prospector.exceptions import FatalProspectorException
from prospector.finder import find_python
from prospector.formatters import FORMATTERS
from prospector.message import Location, Message
from prospector.tools.utils import capture_output

__all__ = (
    "Prospector",
    "main",
)


class Prospector(object):
    def __init__(self, config):
        self.config = config
        self.summary = None
        self.messages = config.messages

    def process_messages(self, found_files, messages):
        for message in messages:
            if self.config.absolute_paths:
                message.to_absolute_path(self.config.workdir)
            else:
                message.to_relative_path(self.config.workdir)
        if self.config.blending:
            messages = blender.blend(messages)

        filepaths = found_files.iter_module_paths(abspath=False)
        return postfilter.filter_messages(filepaths, self.config.workdir, messages)

    def execute(self):

        summary = {
            "started": datetime.now(),
        }
        summary.update(self.config.get_summary_information())

        found_files = find_python(
            self.config.ignores,
            self.config.paths,
            self.config.explicit_file_mode,
            self.config.workdir,
        )

        # Run the tools
        messages = []
        for tool in self.config.get_tools(found_files):
            for name, cls in tools.TOOLS.items():
                if cls == tool.__class__:
                    toolname = name
                    break
            else:
                toolname = "Unknown"

            try:
                # Tools can output to stdout/stderr in unexpected places, for example,
                # pep257 emits warnings about __all__ and as pyroma exec's the setup.py
                # file, it will execute any print statements in that, etc etc...
                with capture_output(hide=not self.config.direct_tool_stdout) as capture:
                    messages += tool.run(found_files)

                    if self.config.include_tool_stdout:
                        loc = Location(self.config.workdir, None, None, None, None)

                        if capture.get_hidden_stderr():
                            msg = "stderr from %s:\n%s" % (
                                toolname,
                                capture.get_hidden_stderr(),
                            )
                            messages.append(Message(toolname, "hidden-output", loc, message=msg))
                        if capture.get_hidden_stdout():
                            msg = "stdout from %s:\n%s" % (
                                toolname,
                                capture.get_hidden_stdout(),
                            )
                            messages.append(Message(toolname, "hidden-output", loc, message=msg))

            except FatalProspectorException as fatal:
                sys.stderr.write(fatal.message)
                sys.exit(2)

            except Exception:  # pylint: disable=broad-except
                if self.config.die_on_tool_error:
                    raise
                else:
                    loc = Location(self.config.workdir, None, None, None, None)
                    msg = (
                        "Tool %s failed to run (exception was raised, re-run prospector with -X to see the stacktrace)"
                        % (toolname,)
                    )
                    message = Message(
                        toolname,
                        "failure",
                        loc,
                        message=msg,
                    )
                    messages.append(message)

        messages = self.process_messages(found_files, messages)

        summary["message_count"] = len(messages)
        summary["completed"] = datetime.now()

        # Timedelta.total_seconds() is not available
        # on Python<=2.6 so we calculate it ourselves
        # See issue #60 and http://stackoverflow.com/a/3694895
        delta = summary["completed"] - summary["started"]
        total_seconds = (delta.microseconds + (delta.seconds + delta.days * 24 * 3600) * 1e6) / 1e6
        summary["time_taken"] = "%0.2f" % total_seconds

        external_config = []
        for tool, configured_by in self.config.configured_by.items():
            if configured_by is not None:
                external_config.append((tool, configured_by))
        if len(external_config) > 0:
            summary["external_config"] = ", ".join(["%s: %s" % info for info in external_config])

        self.summary = summary
        self.messages = self.messages + messages

    def get_summary(self):
        return self.summary

    def get_messages(self):
        return self.messages

    def print_messages(self):
        output_reports = self.config.get_output_report()

        for report in output_reports:
            output_format, output_files = report
            self.summary["formatter"] = output_format
            formatter = FORMATTERS[output_format](self.summary, self.messages, self.config.profile)
            if not output_files:
                self.write_to(formatter, sys.stdout)
            for output_file in output_files:
                with open(output_file, "w+") as target:
                    self.write_to(formatter, target)

    def write_to(self, formatter, target):
        # Produce the output
        target.write(
            formatter.render(
                summary=not self.config.messages_only,
                messages=not self.config.summary_only,
                profile=self.config.show_profile,
            )
        )
        target.write("\n")


def get_parser():
    """
    This is a helper method to return an argparse parser, to
    be used with the Sphinx argparse plugin for documentation.
    """
    manager = cfg.build_manager()
    source = cfg.build_command_line_source(prog="prospector", description=None)
    return source.build_parser(manager.settings, None)


def main():
    # Get our configuration
    config = ProspectorConfig()

    paths = config.paths
    if len(paths) > 1 and not all([os.path.isfile(path) for path in paths]):
        sys.stderr.write("\nIn multi-path mode, all inputs must be files, " "not directories.\n\n")
        get_parser().print_usage()
        sys.exit(2)

    # Make it so
    prospector = Prospector(config)
    prospector.execute()
    prospector.print_messages()

    if config.exit_with_zero_on_success():
        # if we ran successfully, and the user wants us to, then we'll
        # exit cleanly
        return 0

    # otherwise, finding messages is grounds for exiting with an error
    # code, to make it easier for bash scripts and similar situations
    # to know if there any errors have been found.
    if len(prospector.get_messages()) > 0:
        return 1
    return 0


if __name__ == "__main__":
    sys.exit(main())



import logging
import os

from prospector.message import Location, Message
from prospector.tools.base import ToolBase

# HACK: pyroma configures logging in its __init__.py so by importing,
# it will change existing logging configuration to DEBUG which causes
# problems with other 3rd party modules as everything now logs to
# stdout...
_old = logging.basicConfig
try:
    logging.basicConfig = lambda **k: None
    from pyroma import projectdata, ratings
except ImportError:
    # raise the Exception
    raise
finally:
    # always restore logging.basicConfig
    logging.basicConfig = _old


PYROMA_ALL_CODES = {
    "Name": "PYR01",
    "Version": "PYR02",
    "VersionIsString": "PYR03",
    "PEPVersion": "PYR04",
    "Description": "PYR05",
    "LongDescription": "PYR06",
    "Classifiers": "PYR07",
    "PythonVersion": "PYR08",
    "Keywords": "PYR09",
    "Author": "PYR10",
    "AuthorEmail": "PYR11",
    "Url": "PYR12",
    "License": "PYR13",
    "LicenceClassifier": "PYR14",
    "ZipSafe": "PYR15",
    "SDist": "PYR16",
    "PackageDocs": "PYR17",
    "ValidREST": "PYR18",
    "BusFactor": "PYR19",
}

PYROMA_CODES = {}
for name, code in PYROMA_ALL_CODES.items():
    if hasattr(ratings, name):
        PYROMA_CODES[getattr(ratings, name)] = code

PYROMA_TEST_CLASSES = [t.__class__ for t in ratings.ALL_TESTS]


class PyromaTool(ToolBase):
    def __init__(self, *args, **kwargs):
        super(PyromaTool, self).__init__(*args, **kwargs)
        self.ignore_codes = ()

    def configure(self, prospector_config, found_files):
        self.ignore_codes = prospector_config.get_disabled_messages("pyroma")

    def run(self, found_files):
        messages = []
        for module in found_files.iter_module_paths(include_ignored=True):
            dirname, filename = os.path.split(module)
            if filename != "setup.py":
                continue

            data = projectdata.get_data(dirname)

            all_tests = [m() for m in PYROMA_TEST_CLASSES]
            for test in all_tests:
                code = PYROMA_CODES.get(test.__class__, "PYRUNKNOWN")

                if code in self.ignore_codes:
                    continue

                passed = test.test(data)
                if passed is False:  # passed can be True, False or None...
                    loc = Location(module, "setup", None, -1, -1)
                    msg = Message("pyroma", code, loc, test.message())
                    messages.append(msg)

        return messages



from __future__ import absolute_import

from prospector.encoding import CouldNotHandleEncoding, read_py_file
from prospector.message import Location, Message, make_tool_error_message
from prospector.tools.base import ToolBase
from pydocstyle.checker import AllError
from pydocstyle.checker import ConventionChecker as PEP257Checker

__all__ = ("Pep257Tool",)


class Pep257Tool(ToolBase):
    def __init__(self, *args, **kwargs):
        super(Pep257Tool, self).__init__(*args, **kwargs)
        self._code_files = []
        self.ignore_codes = ()

    def configure(self, prospector_config, found_files):
        self.ignore_codes = prospector_config.get_disabled_messages("pep257")

    def run(self, found_files):
        messages = []

        checker = PEP257Checker()

        for code_file in found_files.iter_module_paths():
            try:
                for error in checker.check_source(read_py_file(code_file), code_file, None):

                    location = Location(
                        path=code_file,
                        module=None,
                        function="",
                        line=error.line,
                        character=0,
                        absolute_path=True,
                    )
                    message = Message(
                        source="pep257",
                        code=error.code,
                        location=location,
                        message=error.message.partition(":")[2].strip(),
                    )
                    messages.append(message)
            except CouldNotHandleEncoding as err:
                messages.append(
                    make_tool_error_message(
                        code_file,
                        "pep257",
                        "D000",
                        message="Could not handle the encoding of this file: %s" % err.encoding,
                    )
                )
                continue
            except AllError as exc:
                # pep257's Parser.parse_all method raises AllError when an
                # attempt to analyze the __all__ definition has failed.  This
                # occurs when __all__ is too complex to be parsed.
                messages.append(
                    make_tool_error_message(
                        code_file,
                        "pep257",
                        "D000",
                        line=1,
                        character=0,
                        message=exc.args[0],
                    )
                )
                continue

        return self.filter_messages(messages)

    def filter_messages(self, messages):
        return [message for message in messages if message.code not in self.ignore_codes]



# flake8: noqa
import setoptconf as soc
import pkg_resources
from prospector.config.datatype import OutputChoice
from prospector.formatters import FORMATTERS
from prospector.tools import DEFAULT_TOOLS, TOOLS

__all__ = ("build_manager",)

_VERSION = pkg_resources.get_distribution("prospector").version


def build_manager():
    manager = soc.ConfigurationManager("prospector")

    manager.add(soc.BooleanSetting("zero_exit", default=False))

    manager.add(soc.BooleanSetting("autodetect", default=True))
    manager.add(soc.ListSetting("uses", soc.String, default=[]))

    manager.add(soc.BooleanSetting("blending", default=True))

    manager.add(soc.BooleanSetting("doc_warnings", default=None))
    manager.add(soc.BooleanSetting("test_warnings", default=None))
    manager.add(soc.BooleanSetting("no_style_warnings", default=None))
    manager.add(soc.BooleanSetting("member_warnings", default=None))
    manager.add(soc.BooleanSetting("full_pep8", default=None))
    manager.add(soc.IntegerSetting("max_line_length", default=None))

    manager.add(soc.BooleanSetting("messages_only", default=False))
    manager.add(soc.BooleanSetting("summary_only", default=False))
    manager.add(
        soc.ListSetting(
            "output_format",
            OutputChoice(sorted(FORMATTERS.keys())),
            default=None,
        )
    )
    manager.add(soc.BooleanSetting("absolute_paths", default=False))

    manager.add(
        soc.ListSetting(
            "tools",
            soc.Choice(sorted(TOOLS.keys())),
            default=None,
        )
    )
    manager.add(soc.ListSetting("with_tools", soc.String, default=[]))
    manager.add(soc.ListSetting("without_tools", soc.String, default=[]))
    manager.add(soc.ListSetting("profiles", soc.String, default=[]))
    manager.add(soc.ListSetting("profile_path", soc.String, default=[]))
    manager.add(
        soc.ChoiceSetting(
            "strictness",
            ["veryhigh", "high", "medium", "low", "verylow"],
            default=None,
        )
    )
    manager.add(soc.BooleanSetting("show_profile", default=False))

    manager.add(soc.BooleanSetting("no_external_config", default=False))
    manager.add(soc.StringSetting("pylint_config_file", default=None))

    manager.add(soc.StringSetting("path", default=None))

    manager.add(soc.ListSetting("ignore_patterns", soc.String, default=[]))
    manager.add(soc.ListSetting("ignore_paths", soc.String, default=[]))

    manager.add(soc.BooleanSetting("die_on_tool_error", default=False))
    manager.add(soc.BooleanSetting("include_tool_stdout", default=False))
    manager.add(soc.BooleanSetting("direct_tool_stdout", default=False))

    return manager


def build_default_sources():
    sources = [
        build_command_line_source(),
        soc.EnvironmentVariableSource(),
        soc.ConfigFileSource(
            (
                ".prospectorrc",
                "setup.cfg",
                "tox.ini",
            )
        ),
        soc.ConfigFileSource(
            (
                soc.ConfigDirectory(".prospectorrc"),
                soc.HomeDirectory(".prospectorrc"),
            )
        ),
    ]

    return sources


def build_command_line_source(prog=None, description="Performs static analysis of Python code"):
    parser_options = {}
    if prog is not None:
        parser_options["prog"] = prog
    if description is not None:
        parser_options["description"] = description

    options = {
        "zero_exit": {
            "flags": ["-0", "--zero-exit"],
            "help": "Prospector will exit with a code of 1 (one) if any messages"
            " are found. This makes automation easier; if there are any"
            " problems at all, the exit code is non-zero. However this behaviour"
            " is not always desirable, so if this flag is set, prospector will"
            " exit with a code of 0 if it ran successfully, and non-zero if"
            " it failed to run.",
        },
        "autodetect": {
            "flags": ["-A", "--no-autodetect"],
            "help": "Turn off auto-detection of frameworks and libraries used."
            " By default, autodetection will be used. To specify"
            " manually, see the --uses option.",
        },
        "uses": {
            "flags": ["-u", "--uses"],
            "help": "A list of one or more libraries or frameworks that the"
            " project uses. Possible values are: django, celery, flask. This will be"
            " autodetected by default, but if autodetection doesn't"
            " work, manually specify them using this flag.",
        },
        "blending": {
            "flags": ["-B", "--no-blending"],
            "help": "Turn off blending of messages. Prospector will merge"
            " together messages from different tools if they represent"
            " the same error. Use this option to see all unmerged"
            " messages.",
        },
        "doc_warnings": {
            "flags": ["-D", "--doc-warnings"],
            "help": "Include warnings about documentation.",
        },
        "test_warnings": {
            "flags": ["-T", "--test-warnings"],
            "help": "Also check test modules and packages.",
        },
        "no_style_warnings": {
            "flags": ["-8", "--no-style-warnings"],
            "help": "Don't create any warnings about style. This disables the"
            " PEP8 tool and similar checks for formatting.",
        },
        "member_warnings": {
            "flags": ["-m", "--member-warnings"],
            "help": "Attempt to warn when code tries to access an attribute of a "
            "class or member of a module which does not exist. This is disabled "
            "by default as it tends to be quite inaccurate.",
        },
        "full_pep8": {
            "flags": ["-F", "--full-pep8"],
            "help": "Enables every PEP8 warning, so that all PEP8 style" " violations will be reported.",
        },
        "max_line_length": {
            "flags": ["--max-line-length"],
            "help": "The maximum line length allowed. This will be set by the strictness if no"
            " value is explicitly specified",
        },
        "messages_only": {
            "flags": ["-M", "--messages-only"],
            "help": "Only output message information (don't output summary" " information about the checks)",
        },
        "summary_only": {
            "flags": ["-S", "--summary-only"],
            "help": "Only output summary information about the checks (don't" "output message information)",
        },
        "output_format": {
            "flags": ["-o", "--output-format"],
            "help": "The output format. Valid values are: %s. This will output to stdout by default, however a target file can be used instead by adding :path-to-output-file, eg, -o json:output.json"
            % (", ".join(sorted(FORMATTERS.keys())),),
        },
        "absolute_paths": {
            "help": "Whether to output absolute paths when referencing files "
            "in messages. By default, paths will be relative to the "
            "project path",
        },
        "tools": {
            "flags": ["-t", "--tool"],
            "help": "A list of tools to run. This lets you set exactly which "
            "tools to run. To add extra tools to the defaults, see "
            "--with-tool. Possible values are: %s. By "
            "default, the following tools will be run: %s"
            % (
                ", ".join(sorted(TOOLS.keys())),
                ", ".join(sorted(DEFAULT_TOOLS)),
            ),
        },
        "with_tools": {
            "flags": ["-w", "--with-tool"],
            "help": "A list of tools to run in addition to the default tools. "
            "To specify all tools explicitly, use the --tool argument. "
            "Possible values are %s." % (", ".join(sorted(TOOLS.keys()))),
        },
        "without_tools": {
            "flags": ["-W", "--without-tool"],
            "help": "A list of tools that should not be run. Useful to turn off "
            "only a single tool from the defaults. "
            "To specify all tools explicitly, use the --tool argument. "
            "Possible values are %s." % (", ".join(sorted(TOOLS.keys()))),
        },
        "profiles": {
            "flags": ["-P", "--profile"],
            "help": "The list of profiles to load. A profile is a certain"
            " 'type' of behaviour for prospector, and is represented"
            " by a YAML configuration file. Either a full path to the YAML"
            " file describing the profile must be provided, or it must be"
            " on the profile path (see --profile-path)",
        },
        "profile_path": {
            "flags": ["--profile-path"],
            "help": "Additional paths to search for profile files. By default this"
            " is the path that prospector will check, and a directory "
            ' called ".prospector" in the path that prospector will check.',
        },
        "show_profile": {
            "flags": ["--show-profile"],
            "help": "Include the computed profile in the summary. This will show what"
            " prospector has decided the overall profile is once all profiles"
            " have been combined and inherited from. This will produce a large"
            " output in most cases so is only useful when trying to debug why"
            " prospector is not behaving like you expect.",
        },
        "strictness": {
            "flags": ["-s", "--strictness"],
            "help": "How strict the checker should be. This affects how"
            " harshly the checker will enforce coding guidelines. The"
            ' default value is "medium", possible values are'
            ' "veryhigh", "high", "medium", "low" and "verylow".',
        },
        "no_external_config": {
            "flags": ["-E", "--no-external-config"],
            "help": "Determines how prospector should behave when"
            " configuration already exists for a tool. By default,"
            " prospector will use existing configuration. This flag"
            " will cause prospector to ignore existing configuration"
            " and use its own settings for every tool. Note that"
            " prospector will always use its own config for tools which"
            " do not have custom configuration.",
        },
        "pylint_config_file": {
            "flags": ["--pylint-config-file"],
            "help": "The path to a pylintrc file to use to configure pylint. Prospector will find"
            " .pylintrc files in the root of the project, but you can use this option to "
            "specify manually where it is.",
        },
        "ignore_patterns": {
            "flags": ["-I", "--ignore-patterns"],
            "help": "A list of paths to ignore, as a list of regular"
            " expressions. Files and folders will be ignored if their"
            " full path contains any of these patterns.",
        },
        "ignore_paths": {
            "flags": ["-i", "--ignore-paths"],
            "help": "A list of file or directory names to ignore. If the"
            " complete name matches any of the items in this list, the"
            " file or directory (and all subdirectories) will be"
            " ignored.",
        },
        "die_on_tool_error": {
            "flags": ["-X", "--die-on-tool-error"],
            "help": "If a tool fails to run, prospector will try to carry on."
            " Use this flag to cause prospector to die and raise the"
            " exception the tool generated. Mostly useful for"
            " development on prospector.",
        },
        "include-tool-stdout": {
            "flags": ["--include-tool-stdout"],
            "help": "There are various places where tools will output warnings to "
            "stdout/stderr, which breaks parsing of JSON output. Therefore while tols "
            "is running, this is suppressed. For developing, it is sometimes useful to "
            "see this. This flag will cause stdout/stderr from a tool to be shown as "
            "a normal message amongst other warnings. See also --direct-tool-stdout",
        },
        "direct-tool-stdout": {
            "flags": ["--direct-tool-stdout"],
            "help": "Same as --include-tool-stdout, except the output will be printed "
            "directly rather than shown as a message.",
        },
        "path": {
            "flags": ["-p", "--path"],
            "help": "The path to a Python project to inspect. Defaults to PWD"
            " if not specified. Note: This command line argument is"
            " deprecated and will be removed in a future update. Please"
            " use the positional PATH argument instead.",
        },
    }

    positional = (
        (
            "checkpath",
            {
                "help": "The path to a Python project to inspect. Defaults to PWD"
                "  if not specified. If multiple paths are specified,"
                "  they must all be files (no directories).",
                "metavar": "PATH",
                "nargs": "*",
            },
        ),
    )

    return soc.CommandLineSource(
        options=options,
        version=_VERSION,
        parser_options=parser_options,
        positional=positional,
    )



from __future__ import absolute_import

import tokenize

from pylint.checkers import BaseTokenChecker
from pylint.interfaces import ITokenChecker


class IndentChecker(BaseTokenChecker):

    __implements__ = (ITokenChecker,)

    name = "indentation"
    msgs = {
        "W0313": (
            "File contains mixed indentation - some lines use tabs, some lines use spaces",
            "indentation-mixture",
            "Used when there are some mixed tabs and spaces in a module.",
        ),
        "W0314": (
            "Line uses %s for indentation, but %s required",
            "incorrect-indentation",
            "Used when the indentation of a line does not match the style required by configuration.",
        ),
    }
    options = (
        (
            "indent-strict-spaces",
            {
                "default": False,
                "type": "yn",
                "metavar": "<boolean>",
                "help": "Enforce using only spaces for indentation",
            },
        ),
        (
            "indent-strict-tabs",
            {
                "default": False,
                "type": "yn",
                "metavar": "<boolean>",
                "help": "Enforce using only tabs for indentation",
            },
        ),
    )

    def process_tokens(self, tokens):
        tab_count = space_count = 0
        line_num = 0

        for token in tokens:
            if token[0] == tokenize.NEWLINE:
                line_num += 1
                line = token[4]
                if line.startswith("\t"):
                    if self.config.indent_strict_spaces:
                        # we have tabs but are configured to only allow spaces
                        self.add_message(
                            "incorrect-indentation",
                            line=line_num,
                            args=("tabs", "spaces"),
                        )
                    tab_count += 1

                if line.startswith(" "):
                    if self.config.indent_strict_tabs:
                        # we have tabs but are configured to only allow spaces
                        self.add_message(
                            "incorrect-indentation",
                            line=line_num,
                            args=("spaces", "tabs"),
                        )
                    space_count += 1

        if tab_count > 0 and space_count > 0:
            # this file has mixed indentation!
            self.add_message("indentation-mixture", line=-1)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['prospector',
 'prospector.config',
 'prospector.formatters',
 'prospector.profiles',
 'prospector.tools',
 'prospector.tools.bandit',
 'prospector.tools.dodgy',
 'prospector.tools.frosted',
 'prospector.tools.mccabe',
 'prospector.tools.mypy',
 'prospector.tools.pep257',
 'prospector.tools.pep8',
 'prospector.tools.profile_validator',
 'prospector.tools.pyflakes',
 'prospector.tools.pylint',
 'prospector.tools.pyroma',
 'prospector.tools.vulture']

package_data = \
{'': ['*'], 'prospector.profiles': ['profiles/*']}

install_requires = \
['PyYAML',
 'dodgy>=0.2.1,<0.3.0',
 'mccabe>=0.6.0,<0.7.0',
 'pep8-naming>=0.3.3,<=0.10.0',
 'pycodestyle>=2.6.0,<2.9.0',
 'pydocstyle>=2.0.0',
 'pyflakes>=2.2.0,<2.4.0',
 'pylint-celery==0.3',
 'pylint-django>=2.4.4,<3.0.0',
 'pylint-flask==0.6',
 'pylint-plugin-utils>=0.6,<0.7',
 'pylint>=2.8.3,<2.12.0',
 'requirements-detector>=0.7,<0.8',
 'setoptconf-tmp>=0.3.1,<0.4.0',
 'toml>=0.10.2,<0.11.0']

extras_require = \
{'with_bandit': ['bandit>=1.5.1'],
 'with_everything': ['bandit>=1.5.1',
                     'frosted>=1.4.1',
                     'vulture>=1.5',
                     'mypy>=0.600',
                     'pyroma>=2.4'],
 'with_frosted': ['frosted>=1.4.1'],
 'with_mypy': ['mypy>=0.600'],
 'with_pyroma': ['pyroma>=2.4'],
 'with_vulture': ['vulture>=1.5']}

entry_points = \
{'console_scripts': ['prospector = prospector.run:main']}

setup_kwargs = {
    'name': 'prospector',
    'version': '1.5.2',
    'description': '',
    'long_description': 'prospector\n==========\n\n.. image:: https://img.shields.io/pypi/v/prospector.svg\n   :target: https://pypi.python.org/pypi/prospector\n   :alt: Latest Version of Prospector\n.. image:: https://travis-ci.org/PyCQA/prospector.svg?branch=master\n   :target: https://travis-ci.org/PyCQA/prospector\n   :alt: Build Status\n.. image:: https://landscape.io/github/landscapeio/prospector/master/landscape.svg?style=flat\n   :target: https://landscape.io/github/landscapeio/prospector/master\n   :alt: Code Health\n.. image:: https://img.shields.io/coveralls/PyCQA/prospector.svg?style=flat\n   :target: https://coveralls.io/r/PyCQA/prospector\n   :alt: Test Coverage\n.. image:: https://readthedocs.org/projects/prospector/badge/?version=latest\n   :target: http://prospector.readthedocs.io/\n   :alt: Documentation\n\n\nAbout\n-----\n\nProspector is a tool to analyse Python code and output information about\nerrors, potential problems, convention violations and complexity.\n\nIt brings together the functionality of other Python analysis tools such as\n`Pylint <http://docs.pylint.org/>`_,\n`pep8 <http://pep8.readthedocs.org/en/latest/>`_,\nand `McCabe complexity <https://pypi.python.org/pypi/mccabe>`_.\nSee the `Supported Tools <http://prospector.readthedocs.io/en/latest/supported_tools.html>`_\ndocumentation section for a complete list.\n\nThe primary aim of Prospector is to be useful \'out of the box\'. A common complaint of other\nPython analysis tools is that it takes a long time to filter through which errors are relevant\nor interesting to your own coding style. Prospector provides some default profiles, which\nhopefully will provide a good starting point and will be useful straight away, and adapts\nthe output depending on the libraries your project uses.\n\nInstallation\n------------\n\nProspector can be installed from PyPI using ``pip`` by running the following command::\n\n    pip install prospector\n\nOptional dependencies for Prospector, such as ``pyroma`` can also be installed by running::\n\n    pip install prospector[with_pyroma]\n\nSome shells (such as ``Zsh``, the default shell of macOS Catalina) require brackets to be escaped::\n\n    pip install prospector\\[with_pyroma\\]\n\nFor a list of all of the optional dependencies, see the optional extras section on the ReadTheDocs\npage on `Supported Tools Extras <https://prospector.readthedocs.io/en/latest/supported_tools.html#optional-extras>`_.\n\nFor local development, `poetry <https://python-poetry.org/>`_ is used. Check out the code, then run::\n\n    poetry install\n\nAnd for extras::\n\n    poetry install -E with_everything\n\nFor more detailed information on installing the tool, see the\n`installation section <http://prospector.readthedocs.io/en/latest/#installation>`_ of the tool\'s main page\non ReadTheDocs.\n\nDocumentation\n-------------\n\nFull `documentation is available at ReadTheDocs <http://prospector.readthedocs.io>`_.\n\nUsage\n-----\n\nSimply run prospector from the root of your project::\n\n    prospector\n\nThis will output a list of messages pointing out potential problems or errors, for example::\n\n    prospector.tools.base (prospector/tools/base.py):\n        L5:0 ToolBase: pylint - R0922\n        Abstract class is only referenced 1 times\n\nOptions\n```````\n\nRun ``prospector --help`` for a full list of options and their effects.\n\nOutput Format\n~~~~~~~~~~~~~\n\nThe default output format of ``prospector`` is designed to be human readable. For parsing\n(for example, for reporting), you can use the ``--output-format json`` flag to get JSON-formatted\noutput.\n\nProfiles\n~~~~~~~~\n\nProspector is configurable using "profiles". These are composable YAML files with directives to\ndisable or enable tools or messages. For more information, read\n`the documentation about profiles <http://prospector.readthedocs.io/en/latest/profiles.html>`_.\n\nIf your code uses frameworks and libraries\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\nOften tools such as pylint find errors in code which is not an error, for example due to attributes of classes being\ncreated at run time by a library or framework used by your project.\nFor example, by default, pylint will generate an error for Django models when accessing ``objects``, as the\n``objects`` attribute is not part of the ``Model`` class definition.\n\nProspector mitigates this by providing an understanding of these frameworks to the underlying tools.\n\nProspector will try to intuit which libraries your project uses by\n`detecting dependencies <https://github.com/landscapeio/requirements-detector>`_ and automatically turning on\nsupport for the requisite libraries. You can see which adaptors were run in the metadata section of the report.\n\nIf Prospector does not correctly detect your project\'s dependencies, you can specify them manually from the commandline::\n\n    prospector --uses django celery\n\nAdditionally, if Prospector is automatically detecting a library that you do not in fact use, you can turn\noff autodetection completely::\n\n    prospector --no-autodetect\n\nNote that as far as possible, these adaptors have been written as plugins or augmentations for the underlying\ntools so that they can be used without requiring Prospector. For example, the Django support is available as a pylint plugin.\n\nStrictness\n~~~~~~~~~~\n\nProspector has a configurable \'strictness\' level which will determine how harshly it searches for errors::\n\n    prospector --strictness high\n\nPossible values are ``verylow``, ``low``, ``medium``, ``high``, ``veryhigh``.\n\nProspector does not include documentation warnings by default, but you can turn\nthis on using the ``--doc-warnings`` flag.\n\npre-commit\n----------\n\nIf you\'d like Prospector to be run automatically when making changes to files in your Git\nrepository, you can install `pre-commit <https://pre-commit.com/>`_ and add the following\ntext to your repositories\' ``.pre-commit-config.yaml``::\n\n    repos:\n    -   repo: https://github.com/PyCQA/prospector\n        rev: 1.1.7 # The version of Prospector to use, at least 1.1.7\n        hooks:\n        -   id: prospector\n\nLicense\n-------\n\nProspector is available under the GPLv2 License.\n',
    'author': 'Carl Crowder',
    'author_email': 'git@carlcrowder.com',
    'maintainer': 'Carl Crowder',
    'maintainer_email': 'git@carlcrowder.com',
    'url': 'http://prospector.readthedocs.io',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0',
}


setup(**setup_kwargs)



import json
import os

import yaml
from prospector.tools import TOOLS

BUILTIN_PROFILE_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), "profiles"))


class ProfileNotFound(Exception):
    def __init__(self, name, profile_path):
        super(ProfileNotFound, self).__init__()
        self.name = name
        self.profile_path = profile_path

    def __repr__(self):
        return "Could not find profile %s; searched in %s" % (
            self.name,
            ":".join(self.profile_path),
        )


class CannotParseProfile(Exception):
    def __init__(self, filepath, parse_error):
        super(CannotParseProfile, self).__init__()
        self.filepath = filepath
        self.parse_error = parse_error

    def get_parse_message(self):
        return "%s\n  on line %s : char %s" % (
            self.parse_error.problem,
            self.parse_error.problem_mark.line,
            self.parse_error.problem_mark.column,
        )

    def __repr__(self):
        return "Could not parse profile found at %s - it is not valid YAML" % self.filepath


class ProspectorProfile(object):
    def __init__(self, name, profile_dict, inherit_order):
        self.name = name
        self.inherit_order = inherit_order

        self.ignore_paths = _ensure_list(profile_dict.get("ignore-paths", []))
        # The 'ignore' directive is an old one which should be deprecated at some point
        self.ignore_patterns = _ensure_list(profile_dict.get("ignore-patterns", []) + profile_dict.get("ignore", []))

        self.output_format = profile_dict.get("output-format")
        self.output_target = profile_dict.get("output-target")
        self.autodetect = profile_dict.get("autodetect")
        self.uses = [
            uses for uses in _ensure_list(profile_dict.get("uses", [])) if uses in ("django", "celery", "flask")
        ]
        self.max_line_length = profile_dict.get("max-line-length")

        # informational shorthands
        self.strictness = profile_dict.get("strictness")
        self.test_warnings = profile_dict.get("test-warnings")
        self.doc_warnings = profile_dict.get("doc-warnings")
        self.member_warnings = profile_dict.get("member-warnings")

        # TODO: this is needed by Landscape but not by prospector; there is probably a better place for it
        self.requirements = _ensure_list(profile_dict.get("requirements", []))
        self.python_targets = _ensure_list(profile_dict.get("python-targets", []))

        for tool in TOOLS.keys():
            conf = {"disable": [], "enable": [], "run": None, "options": {}}
            conf.update(profile_dict.get(tool, {}))

            if self.max_line_length is not None and tool in ("pylint", "pep8"):
                conf["options"]["max-line-length"] = self.max_line_length

            setattr(self, tool, conf)

    def get_disabled_messages(self, tool_name):
        disable = getattr(self, tool_name)["disable"]
        enable = getattr(self, tool_name)["enable"]
        return list(set(disable) - set(enable))

    def is_tool_enabled(self, name):
        return getattr(self, name).get("run")

    def list_profiles(self):
        # this profile is itself included
        return self.inherit_order

    def as_dict(self):
        out = {
            "ignore-paths": self.ignore_paths,
            "ignore-patterns": self.ignore_patterns,
            "output-format": self.output_format,
            "output-target": self.output_target,
            "autodetect": self.autodetect,
            "uses": self.uses,
            "max-line-length": self.max_line_length,
            "member-warnings": self.member_warnings,
            "doc-warnings": self.doc_warnings,
            "test-warnings": self.test_warnings,
            "strictness": self.strictness,
            "requirements": self.requirements,
            "python-targets": self.python_targets,
        }
        for tool in TOOLS.keys():
            out[tool] = getattr(self, tool)
        return out

    def as_json(self):
        return json.dumps(self.as_dict())

    def as_yaml(self):
        return yaml.safe_dump(self.as_dict())

    @staticmethod
    def load(name_or_path, profile_path, allow_shorthand=True, forced_inherits=None):
        # First simply load all of the profiles and those that it explicitly inherits from
        data, inherits = _load_and_merge(
            name_or_path,
            profile_path,
            allow_shorthand,
            forced_inherits=forced_inherits or [],
        )
        return ProspectorProfile(name_or_path, data, inherits)


def _is_valid_extension(filename):
    ext = os.path.splitext(filename)[1]
    return ext in (".yml", ".yaml")


def _load_content(name_or_path, profile_path):
    filename = None

    if _is_valid_extension(name_or_path):
        for path in profile_path:
            filepath = os.path.join(path, name_or_path)
            if os.path.exists(filepath):
                # this is a full path that we can load
                filename = filepath
                break

        if filename is None:
            raise ProfileNotFound(name_or_path, profile_path)
    else:
        for path in profile_path:
            for ext in ("yml", "yaml"):
                filepath = os.path.join(path, "%s.%s" % (name_or_path, ext))
                if os.path.exists(filepath):
                    filename = filepath
                    break

        if filename is None:
            raise ProfileNotFound(name_or_path, profile_path)

    with open(filename) as fct:
        try:
            return yaml.safe_load(fct) or {}
        except yaml.parser.ParserError as parse_error:
            raise CannotParseProfile(filename, parse_error)


def _ensure_list(value):
    if isinstance(value, list):
        return value
    return [value]


def _simple_merge_dict(priority, base):
    out = dict(base.items())
    out.update(dict(priority.items()))
    return out


def _merge_tool_config(priority, base):
    out = dict(base.items())
    for key, value in priority.items():
        # pep8 has extra 'full' and 'none' options
        # pylint has extra 'load-plugins' option
        if key in ("run", "full", "none", "load-plugins"):
            out[key] = value
        elif key in ("options",):
            out[key] = _simple_merge_dict(value, base.get(key, {}))

    # anything enabled in the 'priority' dict is removed
    # from 'disabled' in the base dict and vice versa
    base_disabled = base.get("disable") or []
    base_enabled = base.get("enable") or []
    pri_disabled = priority.get("disable") or []
    pri_enabled = priority.get("enable") or []

    out["disable"] = list(set(pri_disabled) | (set(base_disabled) - set(pri_enabled)))
    out["enable"] = list(set(pri_enabled) | (set(base_enabled) - set(pri_disabled)))

    return out


def _merge_profile_dict(priority, base):
    # copy the base dict into our output
    out = dict(base.items())

    for key, value in priority.items():
        if key in (
            "strictness",
            "doc-warnings",
            "test-warnings",
            "member-warnings",
            "output-format",
            "autodetect",
            "max-line-length",
        ):
            # some keys are simple values which are overwritten
            out[key] = value
        elif key in (
            "ignore",
            "ignore-patterns",
            "ignore-paths",
            "uses",
            "requirements",
            "python-targets",
            "output-target",
        ):
            # some keys should be appended
            out[key] = _ensure_list(value) + _ensure_list(base.get(key, []))
        elif key in TOOLS.keys():
            # this is tool config!
            out[key] = _merge_tool_config(value, base.get(key, {}))

    return out


def _determine_strictness(profile_dict, inherits):
    for profile in inherits:
        if profile.startswith("strictness_"):
            return None, False

    strictness = profile_dict.get("strictness")
    if strictness is None:
        return None, False
    return ("strictness_%s" % strictness), True


def _determine_pep8(profile_dict):
    pep8 = profile_dict.get("pep8", {})
    if pep8.get("full", False):
        return "full_pep8", True
    elif pep8.get("none", False):
        return "no_pep8", True
    return None, False


def _determine_doc_warnings(profile_dict):
    doc_warnings = profile_dict.get("doc-warnings")
    if doc_warnings is None:
        return None, False
    return ("doc_warnings" if doc_warnings else "no_doc_warnings"), True


def _determine_test_warnings(profile_dict):
    test_warnings = profile_dict.get("test-warnings")
    if test_warnings is None:
        return None, False
    return (None if test_warnings else "no_test_warnings"), True


def _determine_member_warnings(profile_dict):
    member_warnings = profile_dict.get("member-warnings")
    if member_warnings is None:
        return None, False
    return ("member_warnings" if member_warnings else "no_member_warnings"), True


def _determine_implicit_inherits(profile_dict, already_inherits, shorthands_found):
    # Note: the ordering is very important here - the earlier items
    # in the list have precedence over the later items. The point of
    # the doc/test/pep8 profiles is usually to restore items which were
    # turned off in the strictness profile, so they must appear first.
    implicit = [
        ("pep8", _determine_pep8(profile_dict)),
        ("docs", _determine_doc_warnings(profile_dict)),
        ("tests", _determine_test_warnings(profile_dict)),
        ("strictness", _determine_strictness(profile_dict, already_inherits)),
        ("members", _determine_member_warnings(profile_dict)),
    ]
    inherits = []

    for shorthand_name, determined in implicit:
        if shorthand_name in shorthands_found:
            continue
        extra_inherits, shorthand_found = determined
        if not shorthand_found:
            continue
        shorthands_found.add(shorthand_name)
        if extra_inherits is not None:
            inherits.append(extra_inherits)

    return inherits, shorthands_found


def _append_profiles(name, profile_path, data, inherit_list, allow_shorthand=False):
    new_data, new_il, _ = _load_profile(name, profile_path, allow_shorthand=allow_shorthand)
    data.update(new_data)
    inherit_list += new_il
    return data, inherit_list


def _load_and_merge(name_or_path, profile_path, allow_shorthand=True, forced_inherits=None):
    # First simply load all of the profiles and those that it explicitly inherits from
    data, inherit_list, shorthands_found = _load_profile(
        name_or_path,
        profile_path,
        allow_shorthand=allow_shorthand,
        forced_inherits=forced_inherits or [],
    )

    if allow_shorthand:
        if "docs" not in shorthands_found:
            data, inherit_list = _append_profiles("no_doc_warnings", profile_path, data, inherit_list)

        if "members" not in shorthands_found:
            data, inherit_list = _append_profiles("no_member_warnings", profile_path, data, inherit_list)

        if "tests" not in shorthands_found:
            data, inherit_list = _append_profiles("no_test_warnings", profile_path, data, inherit_list)

        if "strictness" not in shorthands_found:
            # if no strictness was specified, then we should manually insert the medium strictness
            for inherit in inherit_list:
                if inherit.startswith("strictness_"):
                    break
            else:
                data, inherit_list = _append_profiles("strictness_medium", profile_path, data, inherit_list)

    # Now we merge all of the values together, from 'right to left' (ie, from the
    # top of the inheritance tree to the bottom). This means that the lower down
    # values overwrite those from above, meaning that the initially provided profile
    # has precedence.
    merged = {}
    for name in inherit_list[::-1]:
        priority = data[name]
        merged = _merge_profile_dict(priority, merged)

    return merged, inherit_list


def _load_profile(
    name_or_path,
    profile_path,
    shorthands_found=None,
    already_loaded=None,
    allow_shorthand=True,
    forced_inherits=None,
):
    # recursively get the contents of the basic profile and those it inherits from
    base_contents = _load_content(name_or_path, profile_path)

    inherit_order = [name_or_path]
    shorthands_found = shorthands_found or set()

    already_loaded = already_loaded or []
    already_loaded.append(name_or_path)

    inherits = _ensure_list(base_contents.get("inherits", []))
    if forced_inherits is not None:
        inherits += forced_inherits

    # There are some 'shorthand' options in profiles which implicitly mean that we
    # should inherit from some of prospector's built-in profiles
    if base_contents.get("allow-shorthand", True) and allow_shorthand:
        extra_inherits, extra_shorthands = _determine_implicit_inherits(base_contents, inherits, shorthands_found)
        inherits += extra_inherits
        shorthands_found |= extra_shorthands

    contents_dict = {name_or_path: base_contents}

    for inherit_profile in inherits:
        if inherit_profile in already_loaded:
            # we already have this loaded and in the list
            continue

        already_loaded.append(inherit_profile)
        new_cd, new_il, new_sh = _load_profile(
            inherit_profile,
            profile_path,
            shorthands_found,
            already_loaded,
            allow_shorthand,
        )
        contents_dict.update(new_cd)
        inherit_order += new_il
        shorthands_found |= new_sh

    # note: a new list is returned here rather than simply using inherit_order to give astroid a
    # clue about the type of the returned object, as otherwise it can recurse infinitely and crash,
    # this meaning that prospector does not run on prospector cleanly!
    return contents_dict, list(inherit_order), shorthands_found



from prospector.exceptions import FatalProspectorException
from prospector.tools.base import ToolBase
from prospector.tools.dodgy import DodgyTool
from prospector.tools.mccabe import McCabeTool
from prospector.tools.pep8 import Pep8Tool
from prospector.tools.pep257 import Pep257Tool
from prospector.tools.profile_validator import ProfileValidationTool
from prospector.tools.pyflakes import PyFlakesTool
from prospector.tools.pylint import PylintTool


def _tool_not_available(name, install_option_name):
    class NotAvailableTool(ToolBase):
        def run(self, _):
            raise FatalProspectorException(
                "\nCannot run tool %s as support was not installed.\n"
                "Please install by running 'pip install prospector[%s]'\n\n" % (name, install_option_name)
            )

    return NotAvailableTool


def _optional_tool(name, package_name=None, tool_class_name=None, install_option_name=None):
    package_name = "prospector.tools.%s" % (package_name or name)
    tool_class_name = tool_class_name or "%sTool" % name.title()
    install_option_name = install_option_name or ("with_%s" % name)

    try:
        tool_package = __import__(package_name, fromlist=[tool_class_name])
    except ImportError:
        tool_class = _tool_not_available(name, install_option_name)
    else:
        tool_class = getattr(tool_package, tool_class_name)

    return tool_class


TOOLS = {
    "dodgy": DodgyTool,
    "mccabe": McCabeTool,
    "pyflakes": PyFlakesTool,
    "pep8": Pep8Tool,
    "pylint": PylintTool,
    "pep257": Pep257Tool,
    "profile-validator": ProfileValidationTool,
    "frosted": _optional_tool("frosted"),
    "vulture": _optional_tool("vulture"),
    "pyroma": _optional_tool("pyroma"),
    "mypy": _optional_tool("mypy"),
    "bandit": _optional_tool("bandit"),
}


DEFAULT_TOOLS = (
    "dodgy",
    "mccabe",
    "pyflakes",
    "pep8",
    "pylint",
    "pep257",
    "profile-validator",
)



from __future__ import absolute_import

import yaml
from prospector.formatters.base import Formatter

__all__ = ("YamlFormatter",)


# pylint: disable=too-few-public-methods
class YamlFormatter(Formatter):
    def render(self, summary=True, messages=True, profile=False):
        output = {}

        if summary:
            output["summary"] = self.summary

        if profile:
            output["profile"] = self.profile.as_dict()

        if messages:
            output["messages"] = [m.as_dict() for m in self.messages]

        return yaml.safe_dump(
            output,
            indent=2,
            default_flow_style=False,
            allow_unicode=True,
        )



import tokenize


class CouldNotHandleEncoding(Exception):
    def __init__(self, path, cause):
        self.path = path
        self.cause = cause


def read_py_file(filepath):
    # see https://docs.python.org/3/library/tokenize.html#tokenize.detect_encoding
    # first just see if the file is properly encoded
    try:
        with open(filepath, "rb") as f:
            tokenize.detect_encoding(f.readline)
    except SyntaxError as err:
        # this warning is issued:
        #   (1) in badly authored files (contains non-utf8 in a comment line)
        #   (2) a coding is specified, but wrong and
        #   (3) no coding is specified, and the default
        #       'utf8' fails to decode.
        #   (4) the encoding specified by a pep263 declaration did not match
        #       with the encoding detected by inspecting the BOM
        raise CouldNotHandleEncoding(filepath, err)

    try:
        with tokenize.open(filepath) as f:
            return f.read()
        # this warning is issued:
        #   (1) if uft-8 is specified, but latin1 is used with something like \x0e9 appearing
        #       (see http://stackoverflow.com/a/5552623)
    except UnicodeDecodeError as err:
        raise CouldNotHandleEncoding(filepath, err)



import os


class Location(object):
    def __init__(self, path, module, function, line, character, absolute_path=True):
        self.path = path
        self._path_is_absolute = absolute_path
        self.module = module or None
        self.function = function or None
        self.line = None if line == -1 else line
        self.character = None if character == -1 else character

    def to_absolute_path(self, root):
        if self._path_is_absolute:
            return
        self.path = os.path.abspath(os.path.join(root, self.path))
        self._path_is_absolute = True

    def to_relative_path(self, root):
        if not self._path_is_absolute:
            return
        self.path = os.path.relpath(self.path, root)
        self._path_is_absolute = False

    def as_dict(self):
        return {
            "path": self.path,
            "module": self.module,
            "function": self.function,
            "line": self.line,
            "character": self.character,
        }

    def __hash__(self):
        return hash((self.path, self.line, self.character))

    def __eq__(self, other):
        return self.path == other.path and self.line == other.line and self.character == other.character

    def __lt__(self, other):
        if self.path == other.path:
            if self.line == other.line:
                return (self.character or -1) < (other.character or -1)
            return (self.line or -1) < (other.line or -1)  # line can be None if it a file-global warning
        return self.path < other.path


class Message(object):
    def __init__(self, source, code, location, message):
        self.source = source
        self.code = code
        self.location = location
        self.message = message

    def to_absolute_path(self, root):
        self.location.to_absolute_path(root)

    def to_relative_path(self, root):
        self.location.to_relative_path(root)

    def as_dict(self):
        return {
            "source": self.source,
            "code": self.code,
            "location": self.location.as_dict(),
            "message": self.message,
        }

    def __repr__(self):
        return "%s-%s" % (self.source, self.code)

    def __eq__(self, other):
        if self.location == other.location:
            return self.code == other.code
        else:
            return False

    def __lt__(self, other):
        if self.location == other.location:
            return self.code < other.code
        return self.location < other.location


def make_tool_error_message(filepath, source, code, message, line=0, character=0, module=None, function=None):
    location = Location(path=filepath, module=module, function=function, line=line, character=character)
    return Message(source=source, code=code, location=location, message=message)



__all__ = ("Formatter",)


# pylint: disable=too-few-public-methods
class Formatter(object):
    def __init__(self, summary, messages, profile):
        self.summary = summary
        self.messages = messages
        self.profile = profile

    def render(self, summary=True, messages=True, profile=False):
        pass



import os
import re
import sre_constants
import sys

from prospector import tools
from prospector.autodetect import autodetect_libraries
from prospector.config import configuration as cfg
from prospector.profiles import AUTO_LOADED_PROFILES
from prospector.profiles.profile import BUILTIN_PROFILE_PATH, CannotParseProfile, ProfileNotFound, ProspectorProfile
from prospector.tools import DEFAULT_TOOLS


class ProspectorConfig(object):
    # There are several methods on this class which could technically
    # be functions (they don't use the 'self' argument) but that would
    # make this module/class a bit ugly.
    # Also the 'too many instance attributes' warning is ignored, as this
    # is a config object and its sole purpose is to hold many properties!
    # pylint:disable=no-self-use,too-many-instance-attributes

    def __init__(self):
        self.config, self.arguments = self._configure_prospector()

        self.paths = self._get_work_path(self.config, self.arguments)
        self.explicit_file_mode = all(map(os.path.isfile, self.paths))
        self.workdir = os.getcwd()

        self.profile, self.strictness = self._get_profile(self.workdir, self.config)
        self.libraries = self._find_used_libraries(self.config, self.profile)
        self.tools_to_run = self._determine_tool_runners(self.config, self.profile)
        self.ignores = self._determine_ignores(self.config, self.profile, self.libraries)
        self.configured_by = {}
        self.messages = []

    def get_tools(self, found_files):
        self.configured_by = {}
        runners = []
        for tool_name in self.tools_to_run:
            tool = tools.TOOLS[tool_name]()
            config_result = tool.configure(self, found_files)
            if config_result is None:
                configured_by = None
                messages = []
            else:
                configured_by, messages = config_result
                if messages is None:
                    messages = []

            self.configured_by[tool_name] = configured_by
            self.messages += messages
            runners.append(tool)
        return runners

    def get_output_report(self):
        # Get the output formatter
        if self.config.output_format is not None:
            output_report = self.config.output_format
        else:
            output_report = [(self.profile.output_format, self.profile.output_target)]

        for index, report in enumerate(output_report):
            if not all(report):
                output_report[index] = (report[0] or "grouped", report[1] or [])

        return output_report

    def _configure_prospector(self):
        # first we will configure prospector as a whole
        mgr = cfg.build_manager()
        config = mgr.retrieve(*cfg.build_default_sources())
        return config, mgr.arguments

    def _get_work_path(self, config, arguments):
        # Figure out what paths we're prospecting
        if config["path"]:
            paths = [self.config["path"]]
        elif arguments["checkpath"]:
            paths = arguments["checkpath"]
        else:
            paths = [os.getcwd()]
        return paths

    def _get_profile(self, path, config):
        # Use the specified profiles
        profile_provided = False
        if len(config.profiles) > 0:
            profile_provided = True
        cmdline_implicit = []

        # if there is a '.prospector.ya?ml' or a '.prospector/prospector.ya?ml' or equivalent landscape config
        # file then we'll include that
        profile_name = None
        if not profile_provided:
            for possible_profile in AUTO_LOADED_PROFILES:
                prospector_yaml = os.path.join(path, possible_profile)
                if os.path.exists(prospector_yaml) and os.path.isfile(prospector_yaml):
                    profile_provided = True
                    profile_name = possible_profile
                    break

        strictness = None

        if profile_provided:
            if profile_name is None:
                profile_name = config.profiles[0]
                extra_profiles = config.profiles[1:]
            else:
                extra_profiles = config.profiles

            strictness = "from profile"
        else:
            # Use the preconfigured prospector profiles
            profile_name = "default"
            extra_profiles = []

        if config.doc_warnings is not None and config.doc_warnings:
            cmdline_implicit.append("doc_warnings")
        if config.test_warnings is not None and config.test_warnings:
            cmdline_implicit.append("test_warnings")
        if config.no_style_warnings is not None and config.no_style_warnings:
            cmdline_implicit.append("no_pep8")
        if config.full_pep8 is not None and config.full_pep8:
            cmdline_implicit.append("full_pep8")
        if config.member_warnings is not None and config.member_warnings:
            cmdline_implicit.append("member_warnings")

        # Use the strictness profile only if no profile has been given
        if config.strictness is not None and config.strictness:
            cmdline_implicit.append("strictness_%s" % config.strictness)
            strictness = config.strictness

        # the profile path is
        #   * anything provided as an argument
        #   * a directory called .prospector in the check path
        #   * the check path
        #   * prospector provided profiles
        profile_path = config.profile_path

        prospector_dir = os.path.join(path, ".prospector")
        if os.path.exists(prospector_dir) and os.path.isdir(prospector_dir):
            profile_path.append(prospector_dir)

        profile_path.append(path)
        profile_path.append(BUILTIN_PROFILE_PATH)

        try:
            forced_inherits = cmdline_implicit + extra_profiles
            profile = ProspectorProfile.load(profile_name, profile_path, forced_inherits=forced_inherits)
        except CannotParseProfile as cpe:
            sys.stderr.write(
                "Failed to run:\nCould not parse profile %s as it is not valid YAML\n%s\n"
                % (cpe.filepath, cpe.get_parse_message())
            )
            sys.exit(1)
        except ProfileNotFound as nfe:
            sys.stderr.write(
                "Failed to run:\nCould not find profile %s. Search path: %s\n" % (nfe.name, ":".join(nfe.profile_path))
            )
            sys.exit(1)
        else:
            return profile, strictness

    def _find_used_libraries(self, config, profile):
        libraries = []

        # Bring in adaptors that we automatically detect are needed
        if config.autodetect and profile.autodetect is True:
            for found_dep in autodetect_libraries(self.workdir):
                libraries.append(found_dep)

        # Bring in adaptors for the specified libraries
        for name in set(config.uses + profile.uses):
            if name not in libraries:
                libraries.append(name)

        return libraries

    def _determine_tool_runners(self, config, profile):
        if config.tools is None:
            # we had no command line settings for an explicit list of
            # tools, so we use the defaults
            to_run = set(DEFAULT_TOOLS)
            # we can also use any that the profiles dictate
            for tool in tools.TOOLS.keys():
                if profile.is_tool_enabled(tool):
                    to_run.add(tool)
        else:
            to_run = set(config.tools)
            # profiles have no say in the list of tools run when
            # a command line is specified

        for tool in config.with_tools:
            to_run.add(tool)

        for tool in config.without_tools:
            if tool in to_run:
                to_run.remove(tool)

        if config.tools is None and len(config.with_tools) == 0 and len(config.without_tools) == 0:
            for tool in tools.TOOLS.keys():
                enabled = profile.is_tool_enabled(tool)
                if enabled is None:
                    enabled = tool in DEFAULT_TOOLS
                if tool in to_run and not enabled:
                    to_run.remove(tool)

        return sorted(list(to_run))

    def _determine_ignores(self, config, profile, libraries):
        # Grab ignore patterns from the options
        ignores = []
        for pattern in config.ignore_patterns + profile.ignore_patterns:
            if pattern is None:
                # this can happen if someone has a profile with an empty ignore-patterns value, eg:
                #
                #  ignore-patterns:
                #  uses: django
                continue
            try:
                ignores.append(re.compile(pattern))
            except sre_constants.error:
                pass

        # Convert ignore paths into patterns
        boundary = r"(^|/|\\)%s(/|\\|$)"
        for ignore_path in config.ignore_paths + profile.ignore_paths:
            ignore_path = str(ignore_path)
            if ignore_path.endswith("/") or ignore_path.endswith("\\"):
                ignore_path = ignore_path[:-1]
            ignores.append(re.compile(boundary % re.escape(ignore_path)))

        # some libraries have further automatic ignores
        if "django" in libraries:
            ignores += [re.compile("(^|/)(south_)?migrations(/|$)")]

        return ignores

    def get_summary_information(self):
        return {
            "libraries": self.libraries,
            "strictness": self.strictness,
            "profiles": ", ".join(self.profile.list_profiles()),
            "tools": self.tools_to_run,
        }

    def exit_with_zero_on_success(self):
        return self.config.zero_exit

    def get_disabled_messages(self, tool_name):
        return self.profile.get_disabled_messages(tool_name)

    def use_external_config(self, _):
        # Currently there is only one single global setting for whether to use
        # global config, but this could be extended in the future
        return not self.config.no_external_config

    def tool_options(self, tool_name):
        tool = getattr(self.profile, tool_name, None)
        if tool is None:
            return {}
        return tool.get("options", {})

    def external_config_location(self, tool_name):
        return getattr(self.config, "%s_config_file" % tool_name, None)

    @property
    def die_on_tool_error(self):
        return self.config.die_on_tool_error

    @property
    def summary_only(self):
        return self.config.summary_only

    @property
    def messages_only(self):
        return self.config.messages_only

    @property
    def blending(self):
        return self.config.blending

    @property
    def absolute_paths(self):
        return self.config.absolute_paths

    @property
    def max_line_length(self):
        return self.config.max_line_length

    @property
    def include_tool_stdout(self):
        return self.config.include_tool_stdout

    @property
    def direct_tool_stdout(self):
        return self.config.direct_tool_stdout

    @property
    def show_profile(self):
        return self.config.show_profile
